def mfcmmand(self, fnumb="", fname="", ext="", **kwargs):
    """Captures field solution options in a command file.

    APDL Command: MFCMMAND

    Parameters
    ----------
    fnumb
        Field number specified by the MFELEM command.

    fname
        Command file name specified for the field number. Defaults to field
        "FNUMB".

    ext
        Extension for Fname. Defaults to .cmd.

    Notes
    -----
    All relevant solution option commands for the specified field are
    written to a file with the extension .cmd. Refer to the commands in the
    following tables in the Command Reference: Analysis Options, Nonlinear
    Options, Dynamic Options, and Load Step Options.

    This command is also valid in PREP7.

    See Multi-field Commands in the Coupled-Field Analysis Guide for a list
    of all ANSYS Multi-field solver commands and their availability for MFS
    and MFX analyses.

    Distributed ANSYS Restriction: This command is not supported in
    Distributed ANSYS.
    """
    command = f"MFCMMAND,{fnumb},{fname},{ext}"
    return self.run(command, **kwargs)


def mfelem(self, fnumb="", itype1="", itype2="", itype3="", itype4="",
           itype5="", itype6="", itype7="", itype8="", itype9="",
           itype10="", **kwargs):
    """Defines a field by grouping element types.

    APDL Command: MFELEM

    Parameters
    ----------
    fnumb
        Field number for a group of element types.

    itype1, itype2, itype3, . . . , itype10
        Element types defined by the ET command.

    Notes
    -----
    You can define up to ten element types per field.

    Define only element types that contain elements in the field. Do not
    include MESH200 because it is a "mesh-only" element that does not
    contribute to the solution.

    This command is also valid in PREP7.

    See Multi-field Commands in the Coupled-Field Analysis Guide for a list
    of all ANSYS Multi-field solver commands and their availability for MFS
    and MFX analyses.

    Distributed ANSYS Restriction: This command is not supported in
    Distributed ANSYS.
    """
    command = f"MFELEM,{fnumb},{itype1},{itype2},{itype3},{itype4},{itype5},{itype6},{itype7},{itype8},{itype9},{itype10}"
    return self.run(command, **kwargs)


def mfem(self, fnumb="", itype1="", itype2="", itype3="", itype4="",
         itype5="", itype6="", itype7="", itype8="", itype9="", itype10="",
         **kwargs):
    """Add more element types to a previously defined field number.

    APDL Command: MFEM

    Parameters
    ----------
    fnumb
        Existing field number defined by the MFELEM command.

    itype1, itype2, itype3, . . . , itype10
        Element types defined by the ET command.

    Notes
    -----
    You can add up to ten element types per MFEM command. This command
    should not be used after an initial solution.

    This command is also valid in PREP7.

    See Multi-field Commands in the Coupled-Field Analysis Guide for a list
    of all ANSYS Multi-field solver commands and their availability for MFS
    and MFX analyses.

    Distributed ANSYS Restriction: This command is not supported in
    Distributed ANSYS.
    """
    command = f"MFEM,{fnumb},{itype1},{itype2},{itype3},{itype4},{itype5},{itype6},{itype7},{itype8},{itype9},{itype10}"
    return self.run(command, **kwargs)


def mfexter(self, fnumb1="", fnumb2="", fnumb3="", fnumb4="", fnumb5="",
            fnumb6="", fnumb7="", fnumb8="", fnumb9="", fnumb10="",
            fnumb11="", fnumb12="", fnumb13="", fnumb14="", fnumb15="",
            fnumb16="", fnumb17="", fnumb18="", fnumb19="", fnumb20="",
            **kwargs):
    """Defines external fields for an ANSYS Multi-field solver analysis.

    APDL Command: MFEXTER

    Parameters
    ----------
    fnumb1, fnumb2, fnumb3, . . . , fnumb20
        External field numbers defined by the MFELEM command.

    Notes
    -----
    This command specifies external field numbers to be used for load
    transfer in an ANSYS Multi-field solver analysis. Use the MFIMPORT
    command to import the external fields.

    Use the MFELEM command to specify external field numbers. Use the
    MFORDER command to specify the solution order for the external fields.

    You can define a maximum of 20 fields.

    This command is also valid in PREP7.

    See Multi-field Commands in the Coupled-Field Analysis Guide for a list
    of all ANSYS Multi-field solver commands and their availability for MFS
    and MFX analyses.

    Distributed ANSYS Restriction: This command is not supported in
    Distributed ANSYS.
    """
    command = f"MFEXTER,{fnumb1},{fnumb2},{fnumb3},{fnumb4},{fnumb5},{fnumb6},{fnumb7},{fnumb8},{fnumb9},{fnumb10},{fnumb11},{fnumb12},{fnumb13},{fnumb14},{fnumb15},{fnumb16},{fnumb17},{fnumb18},{fnumb19},{fnumb20}"
    return self.run(command, **kwargs)


def mffname(self, fnumb="", fname="", **kwargs):
    """Specifies a file name for a field in an ANSYS Multi-field solver

    APDL Command: MFFNAME
    analysis.

    Parameters
    ----------
    fnumb
        Field number specified by the MFELEM command.

    fname
        File name. Defaults to field "FNUMB".

    Notes
    -----
    All files created for the field will have this file name with the
    appropriate extensions.

    This command is also valid in PREP7.

    See Multi-field Commands in the Coupled-Field Analysis Guide for a list
    of all ANSYS Multi-field solver commands and their availability for MFS
    and MFX analyses.

    Distributed ANSYS Restriction: This command is not supported in
    Distributed ANSYS.
    """
    command = f"MFFNAME,{fnumb},{fname}"
    return self.run(command, **kwargs)



"""
These PREP7 commands define the type of elements to be used in the model.
"""
from typing import Optional, Union

from ansys.mapdl.core._commands.parse import parse_e, parse_et
from ansys.mapdl.core.mapdl_types import MapdlInt, MapdlFloat


def dof(self, lab1="", lab2="", lab3="", lab4="", lab5="", lab6="",
        lab7="", lab8="", lab9="", lab10="", **kwargs):
    """Adds degrees of freedom to the current DOF set.

    APDL Command: DOF

    Parameters
    ----------
    lab1, lab2, lab3, . . . , lab10
        Valid labels are: UX, UY, UZ (structural displacements); ROTX,
        ROTY, ROTZ (structural rotations); TEMP, TBOT, TE2, TE3, . . .,
        TTOP (temperatures);  PRES (pressure);  VOLT (voltage); MAG
        (magnetic scalar potential);  AX, AY, AZ (magnetic vector
        potentials);  CURR (current);  EMF (electromotive force drop); CONC
        (concentration); DELETE.

    Notes
    -----
    The degree of freedom (DOF) set for the model is determined from all
    element types defined. This command may be used to add to the current
    set.  The ALL label may be used on some commands to represent all
    labels of the current degree of freedom set for the model.  Issue the
    DOF command with no arguments to list the current set.  Use the DELETE
    label to delete any previously added DOFs and return to the default DOF
    set.
    """
    command = "DOF,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s" % (str(lab1), str(lab2), str(
        lab3), str(lab4), str(lab5), str(lab6), str(lab7), str(lab8), str(lab9), str(lab10))
    return self.run(command, **kwargs)


def elbow(self, transkey="", tol="", dof="", cons1="", cons2="", cons3="",
          cons4="", **kwargs):
    """APDL Command: ELBOW

    Specifies degrees of freedom to be coupled for end release and applies
    section constraints to elbow elements.

    Parameters
    ----------
    transkey
        Pipe-to-elbow transition flag:

        OFF - Do not automatically transition pipes to elbows. (This behavior is the
              default.)

        ON - Automatically convert straight PIPE289 elements to ELBOW290 elements where it
             is beneficial. The program converts elements in transition
             regions where curved ELBOW290 elements are connected to
             straight PIPE289 elements.

    tol
        Angle tolerance (in degrees) between adjacent ELBOW290 elements.
        The default value is 20. A value of -1 specifies all selected
        ELBOW290 elements.

    dof
        Degrees of freedom to couple:

        ALL - Couple all nodal degrees of freedom (UX, UY, UZ, ROTX, ROTY, and ROTZ). This
              behavior is the default.

        BALL - Create ball joints (equivalent to releasing ROTX, ROTY, and ROTZ).

    cons1, cons2, cons3, cons4
        Section degrees of freedoms to constrain. If Cons1 through Cons4
        are unspecified, no section constraints are applied:

        SECT  - All section deformation

        SE - Section radial expansion

        SO - Section ovalization

        SW - Section warping

        SRA - Local shell normal rotation about cylindrical axis t2

        SRT - Local shell normal rotation about cylindrical axis t1

    Notes
    -----
    The ELBOW command specifies end releases and section constraints for
    ELBOW290 elements and converts straight PIPE289 elements to ELBOW290
    elements.

    Curved PIPE289 elements are not converted to ELBOW290 elements.

    ELBOW290 elements are generated only if there are existing ELBOW290
    elements in the curved areas.

    The command works on currently selected nodes and elements. It creates
    end releases on any two connected elbow elements whose angle at
    connection exceeds the specified tolerance. From within the GUI, the
    Picked node option generates an end release and section constraints at
    the selected node regardless of the angle of connection (that is, the
    angle tolerance [TOL ] is set to -1).

    Elbow and pipe elements must share the same section ID in order for the
    pipe-to-elbow transition to occur.

    To list the elements altered by the ELBOW command, issue an ELIST
    command.

    To list the coupled sets generated by the ELBOW command, issue a CPLIST
    command.

    To list the section constraints generated by the ELBOW command, issue a
    DLIST command.
    """
    command = "ELBOW,%s,%s,%s,%s,%s,%s,%s" % (str(transkey), str(
        tol), str(dof), str(cons1), str(cons2), str(cons3), str(cons4))
    return self.run(command, **kwargs)


def et(self, itype: MapdlInt = "", ename: Union[str, int] = "",
       kop1: MapdlInt = "", kop2: MapdlInt = "", kop3: MapdlInt =
       "", kop4: MapdlInt = "", kop5: MapdlInt = "", kop6:
       MapdlInt = "", inopr: MapdlInt = "", **kwargs) -> Optional[int]:
    """Define a local element type from the element library.

    APDL Command: ET

    Parameters
    ----------
    itype
        Arbitrary local element type number. Defaults to 1 +
        current maximum.

    ename
        Element name (or number) as given in the element library
        in Chapter 4 of the Element Reference. The name consists
        of a category prefix and a unique number, such as PIPE288.
        The category prefix of the name (PIPE for the example) may
        be omitted but is displayed upon output for clarity. If
        ``ename=0``, the element is defined as a null element.

    kop1, kop2, kop3, kop4, kop5, kop6
        KEYOPT values (1 through 6) for this element, as described
        in the Element Reference.

    inopr
        If 1, suppress all element solution printout for this
        element type.

    Examples
    --------
    Define an element type.  Allow MAPDL to pick your the element
    type.

    >>> etype_num = mapdl.et('', 'SURF154')
    >>> etype_num
    1

    Define an element type while specifying the element type
    number.

    >>> etype_num = mapdl.et(2, 'SOLID186')
    >>> etype_num
    2

    Notes
    -----
    The ET command selects an element type from the element
    library and establishes it as a local element type for the
    current model.  Information derived from the element type is
    used for subsequent commands, so the ET command(s) should be
    issued early. (The Element Reference describes the available
    elements.)

    A special option, ``ename=0``, permits the specified element
    type to be ignored during solution without actually removing
    the element from the model. Ename may be set to zero only
    after the element type has been previously defined with a
    nonzero Ename.  The preferred method of ignoring elements is
    to use the select commands (such as ESEL).

    KOPn are element option keys. These keys (referred to as
    KEYOPT(n)) are used to turn on certain element options for
    this element. These options are listed under "KEYOPT" in the
    input table for each element type in the Element Reference.
    KEYOPT values include stiffness formulation options, printout
    controls, and various other element options. If KEYOPT(7) or
    greater is needed, input their values with the KEYOPT command.

    The ET command only defines an element type local to your
    model (from the types in the element library). The TYPE or
    similar [KATT, LATT, AATT, or VATT] command must be used to
    point to the desired local element type before meshing.

    To activate the ANSYS program's LS-DYNA explicit dynamic
    analysis capability, use the ET command
    to choose an element that works only with LS-DYNA (such as
    SHELL163).
    """
    command = f"ET,{itype},{ename},{kop1},{kop2},{kop3},{kop4}," \
              f"{kop5},{kop6},{inopr}"
    return parse_et(self.run(command, **kwargs))


def etchg(self, cnv: str = "", **kwargs) -> Optional[str]:
    """Changes element types to their corresponding types.

    APDL Command: ETCHG

    Parameters
    ----------
    cnv
        Converts the element types to the corresponding type.
        Valid labels
        are:

        ETI - Explicit to Implicit

        ITE - Implicit to Explicit

        TTE - Thermal to Explicit

        TTS - Thermal to Structural

        STT - Structural to Thermal

        MTT - Magnetic to Thermal

        FTS - Fluid to Structural

        ETS - Electrostatic to Structural

        ETT - Electrical to Thermal

    Notes
    -----
    Changes the currently defined element types to their
    corresponding types.  Elements without a companion element
    (listed above) are not switched and should be switched with
    the ET command to an appropriate element type or to a null
    element. The KEYOPT values for the switched element types are
    reset to zero or to their default values. You must
    check these values to see if they are still meaningful.
    Additionally, if Cnv = ETI, ITE, or TTE, all real constants
    are set to zero.

    If Cnv = ITE, you will need to choose a material model that
    corresponds to your previously-defined material properties.
    If working interactively, you will be prompted to do so.
    """
    command = f"ETCHG,{cnv}"
    return self.run(command, **kwargs)


def etcontrol(self, eltech: str = "", eldegene: str = "",
              **kwargs) -> Optional[str]:
    """Control the element technologies used in element
    formulation (for applicable elements).

    APDL Command: ETCONTROL

    Parameters
    ----------
    eltech
        Element technology control.  One of the following:

        - ``"SUGGESTION"`` : The program offers a suggestion for the
          best element technology before solving.  If necessary,
          mixed u-P (KEYOPT(6)) is also included and reset. This
          behavior is the default.

        - ``"SET"`` : The program informs you of the best settings
          and resets any applicable KEYOPT settings
          automatically. This action overrides any previous manual
          settings.

        - ``"OFF"`` : Deactivates automatic selection of element
          technology. No suggestions are issued, and no
          automatic resetting occurs.

    eldegene
        Element degenerated shape control.  One of the following:

        - ``"ON"`` - If element shapes are degenerated, the
          degenerated shape function is employed and enhanced
          strain, simplified enhanced strain, and B-bar formulations
          are turned off (default).

        - ``"OFF"`` - If element shapes are degenerated, regular
          shape functions are still used, and the specified element
          technologies (e.g., enhanced strain, B-bar, uniform
          reduced integration) are still used.

    Examples
    --------
    Enable element tech control and degenerated shape control.

    >>> mapdl.et(1, 'SOLID186')
    >>> output = mapdl.etcontrol(eltech='SET', eldegene='ON')
    >>> print(output)
    ELEMENT TECHNOLOGY CONTROL PARAMETER FOR APPLICABLE ELEMENTS = SET.
     DEGENERATED ELEMENT SHAPE CONTROL PARAMETER FOR APPLICABLE ELEMENTS = ON.

    Notes
    -----
    The command default is ``mapdl.etcontrol('SUGGESTION', 'ON')``

    This command is valid for elements SHELL181, PLANE182,
    PLANE183, SOLID185, SOLID186, SOLID187, BEAM188, BEAM189,
    SHELL208, SHELL209, PLANE223, SOLID226, SOLID227, REINF264,
    SOLID272, SOLID273, SHELL281, SOLID285, PIPE288, PIPE289,
    ELBOW290.

    For more information, see Automatic Selection of Element
    Technologies and Formulations in the Element Reference.
    """
    return self.run(f"ETCONTROL,{eltech},{eldegene}", **kwargs)


def etdele(self, ityp1: Union[str, int] = "", ityp2: MapdlInt = "",
           inc: MapdlInt = "", **kwargs) -> Optional[str]:
    """APDL Command: ETDELE

    Deletes element types.

    Examples
    --------
    Create and delete an element type.

    >>> mapdl.et(1, 'SOLID186')
    >>> mapdl.etdele(1)

    Parameters
    ----------
    ityp1, ityp2, inc
        Deletes element types from ``ityp1`` to ``ityp2``
        (defaults to ``ityp1``) in steps of ``inc`` (defaults to
        1). If ``ityp1='ALL'``, ``ityp2`` and ``inc`` are ignored
        and all element types are deleted.  Element types are
        defined with the ``et`` command.
    """
    command = f"ETDELE,{ityp1},{ityp2},{inc}"
    return self.run(command, **kwargs)


def etlist(self, ityp1: MapdlInt = "", ityp2: MapdlInt = "",
           inc: MapdlInt = "", **kwargs) -> Optional[str]:
    """Lists currently defined element types.

    APDL Command: ETLIST

    Parameters
    ----------
    ityp1, ityp2, inc
        Lists element types from ITYP1 to ITYP2 (defaults to
        ITYP1) in steps of INC (defaults to 1). If ITYP1 = ALL (
        default), ITYP2 and INC are ignored and all element types
        are listed.

    Notes
    -----
    This command is valid in any processor.
    """
    command = f"ETLIST,{ityp1},{ityp2},{inc}"
    return self.run(command, **kwargs)


def keyopt(self, itype="", knum="", value="", **kwargs):
    """APDL Command: KEYOPT

    Sets element key options.

    Parameters
    ----------
    itype
        Element type number as defined on the ET command. The label GCN is
        also valid input for general contact elements (see Notes).

    knum
        Number of the KEYOPT to be defined (KEYOPT(KNUM)).

    value
        Value of this KEYOPT.

    Notes
    -----
    Alternative to inputting KEYOPT values on ET command.  Must be used if
    KEYOPT(7) or greater values are to be input. ITYPE must first be
    defined with the ET command.

    Specify ITYPE = GCN to set element key options for all contact elements
    types used in any existing general contact definitions (that is,
    contact elements having a real constant set number = 0).
    """
    command = "KEYOPT,%s,%s,%s" % (str(itype), str(knum), str(value))
    return self.run(command, **kwargs)


def nsvr(self, itype="", nstv="", **kwargs):
    """APDL Command: NSVR

    Defines the number of variables for user-programmable element options.

    Parameters
    ----------
    itype
        Element type number as defined on the ET command.

    nstv
        Number of extra state variables to save (must be no more than 840).

    Notes
    -----
    Defines the number of extra variables that need to be saved for user-
    programmable (system-dependent) element options, e.g., material laws
    through user subroutine USERPL.  ITYPE must first be defined with the
    ET command.
    """
    command = "NSVR,%s,%s" % (str(itype), str(nstv))
    return self.run(command, **kwargs)



def pltrac(self, analopt="", item="", comp="", trpnum="", name="",
           mxloop="", toler="", option="", escl="", mscl="", **kwargs):
    """Displays a particle flow or charged particle trace on an element

    APDL Command: PLTRAC
    display.

    Parameters
    ----------
    analopt
        Analysis option

        FLUID - Particle trace in fluid flow (default)

        ELEC - Particle trace in electric field

        MAGN - Particle trace in magnetic field

        EMAG - Particle trace in presence of both electric and magnetic fields

    item
        Label identifying the item to be contoured.  Valid item labels are
        shown in Table 222: PLTRAC - Valid Item and Component Labels
        below.  Some items also require a component label.  If Item is
        blank, display only the path trajectory.

    comp
        Component of the item (if required).  Valid component labels are
        shown in Table 222: PLTRAC - Valid Item and Component Labels below.

    trpnum
        Trace point number for storing trajectory data for use with PATH
        logic. Defaults to 0 (no trajectory path data is stored for further
        processing with PATH logic).

    name
        Name of prefix of array variable. Defaults to TRAC. NamePOIN stores
        trajectory path points for trace point number TRPNum. If Analopt =
        ELEC, MAGN, or EMAG, two additional array parameters, NameDATA and
        NameLABL, store trajectory path data and labels for the same
        TRPNum.

    mxloop
        Maximum number of loops traced by a particle.  Defaults to 25 for
        Opt = FLUID; otherwise, defaults to 1000.

    toler
        Length tolerance used for particle trajectory geometry calculation.
        Valid only for Analopt = ELEC, MAGN, or EMAG.  If particle trace
        appears to terminate inside an element, adjusting the length
        tolerance may be necessary.  Defaults to 1.0 x 10-8.

    option
        Flow trace option:

        0 - Use the undeformed mesh for computing the flow trace.

        1 - Use the deformed mesh for computing the flow trace.

    escl
        Electric field scale factor. Setting this scale factor affects only
        the tracing, not the field solution results. A negative factor
        corresponds to the opposite vector direction. Valid only for
        Analopt = ELEC or EMAG. Defaults to 1.

    mscl
        Magnetic field scale factor. Setting this scale factor affects only
        the tracing, not the field solution results. A negative factor
        corresponds to the opposite vector direction. Valid only for
        Analopt = MAGN or EMAG. Defaults to 1.

    Notes
    -----
    For a specified item, the variation of the item is displayed along the
    particle trace as a color-contoured ribbon.  The TRPOIN command must be
    used to define a point on the trajectory path.  Multiple traces may be
    displayed simultaneously by defining multiple trace points.  Issue the
    TRPLIS command to list the current tracing points.  Issue the TRPDEL
    command to delete tracing points defined earlier.   Use the PAPUT
    command with the POIN option to retrieve the particle trajectory points
    as path points.

    The model must be 3-D for the ELEC, MAGN, and EMAG analysis options.

    Three array parameters are created at the time of the particle trace:
    TRACPOIN, TRACDATA and TRACLABL. These array parameters can be used to
    put the particle velocity and the elapsed time into path form.   The
    procedure to put the arrays into a path named PATHNAME is as follows:

    Not used if Analopt = FLUID.  If working in the GUI, use the "All
    information" option to retrieve information from all three arrays at
    once.

    If OPTION is set to 1, the deformed mesh is based on the displacement
    degrees of freedom UX, UY, and UZ, which must be available in the load
    step.

    Table: 222:: : PLTRAC - Valid Item and Component Labels

    See the Basic Analysis Guide for more information on particle flow and
    charged particle traces.  See Animation in the Basic Analysis Guide for
    information on particle trace animation.

    Distributed ANSYS Restriction: This command is not supported in
    Distributed ANSYS.
    """
    command = f"PLTRAC,{analopt},{item},{comp},{trpnum},{name},{mxloop},{toler},{option},{escl},{mscl}"
    return self.run(command, **kwargs)


def trpdel(self, ntrp1="", ntrp2="", trpinc="", **kwargs):
    """Deletes particle flow or charged particle trace points.

    APDL Command: TRPDEL

    Parameters
    ----------
    ntrp1, ntrp2, trpinc
        Delete points from NTRP1 to NTRP2 (defaults to NTRP1) in steps of
        TRPINC (defaults to 1).  If NTRP1 = ALL, NTRP2 and TRPINC are
        ignored and all trace points are deleted.  If NTRP1 = P, graphical
        picking is enabled and all remaining command fields are ignored
        (valid only in the GUI).

    Notes
    -----
    Deletes particle flow or charged particle trace points defined with the
    TRPOIN command.

    Distributed ANSYS Restriction: This command is not supported in
    Distributed ANSYS.
    """
    command = f"TRPDEL,{ntrp1},{ntrp2},{trpinc}"
    return self.run(command, **kwargs)


def trplis(self, ntrp1="", ntrp2="", trpinc="", opt="", **kwargs):
    """Lists the particle flow or charged particle trace points.

    APDL Command: TRPLIS

    Parameters
    ----------
    ntrp1, ntrp2, trpinc
        List points from NTRP1 to NTRP2 (defaults to NTRP1) in steps of
        TRPINC (defaults to 1).  If NTRP1 = ALL, NTRP2 and TRPINC are
        ignored and all trace points are listed.  If NTRP1 = P, graphical
        picking is enabled and all remaining command fields are ignored
        (valid only in the GUI).

    opt
        Opt = LOC lists the trace point number location (X, Y, Z). Default.

    Notes
    -----
    Lists the particle flow or charged particle trace points in the active
    display coordinate system [DSYS].  Trace points are defined with the
    TRPOIN command.

    Distributed ANSYS Restriction: This command is not supported in
    Distributed ANSYS.
    """
    command = f"TRPLIS,{ntrp1},{ntrp2},{trpinc},{opt}"
    return self.run(command, **kwargs)


def trpoin(self, x="", y="", z="", vx="", vy="", vz="", chrg="", mass="",
           **kwargs):
    """Defines a point through which a particle flow or charged particle trace

    APDL Command: TRPOIN
    will travel.

    Parameters
    ----------
    x, y, z
        Coordinate location of the trace point (in the active coordinate
        system).  If X = P, graphical picking is enabled and all remaining
        command fields are ignored (valid only in the GUI).

    vx, vy, vz
        Particle velocities in the X, Y and Z directions (in the active
        coordinate system).

    chrg
        Particle charge.

    mass
        Particle mass.

    Notes
    -----
    Defines a point through which a particle flow or charged particle trace
    [PLTRAC] will travel.  Multiple points (50 maximum) may be defined
    which will result in multiple flow traces.  Use TRPLIS to list the
    currently defined trace points and TRPDEL to delete trace points.

    The VX, VY, VZ, CHRG, and MASS arguments only apply to charged
    particles.

    Distributed ANSYS Restriction: This command is not supported in
    Distributed ANSYS.
    """
    command = f"TRPOIN,{x},{y},{z},{vx},{vy},{vz},{chrg},{mass}"
    return self.run(command, **kwargs)



"""
These DATABASE commands can be used to initialize the database, save
it to a file, or annotate it with titles and systems of units.
"""


def resume(self, fname="", ext="", nopar="", knoplot="", **kwargs):
    """Resumes the database from the database file.

    APDL Command: RESUME

    Parameters
    ----------
    fname
        File name and directory path (248 characters maximum,
        including the characters needed for the directory path).
        An unspecified directory path defaults to the working
        directory; in this case, you can use all 248 characters
        for the file name.

    ext
        Filename extension (eight-character maximum).

    nopar
        Parameter resume key:

        0 - All data in the database, including the scalar
            parameters, are replaced with the data saved on
            File.DB (default).

        1 - All data in the database, except the scalar
            parameters, are replaced with the data saved on
            File.DB.

    knoplot
        If equal to 1, will suppress automatic plot. Otherwise, if
        the GUI is on and this RESUME command was not read from a
        file, the selected elements from Fname are plotted. (If
        there are no selected elements, selected nodes are
        plotted. If no nodes, volumes; if no volumes, areas; if no
        areas, lines; if no lines, keypoints. If there are no
        selected keypoints, the screen is erased.)

    Notes
    -----
    The RESUME command resumes a database file into the ANSYS program. The
    command causes the database file (File.DB) to be read, thereby
    resetting the database (including any geometry settings) either a) as
    it was at the last SAVE command, or b) as it was saved with the last
    /EXIT command, whichever was last.

    For multiple load step analyses (because only the data for one load
    step at a time may reside in the database), the load step data restored
    to the database will correspond to the load step data written when the
    save occurred.

    If the database file was saved in another ANSYS, Inc. product, it may
    contain element type and KEYOPT specifications which are invalid in the
    resuming product. Immediately after the database resume is completed,
    you should redefine these invalid element types and KEYOPT settings to
    valid ones (ET, KEYOPT).

    The NOPAR = 1 option should not be used if array parameters are
    defined, as existing array parameters might be redefined with arbitrary
    values. For a more general method of preventing the replacement of both
    scalar and array parameters, see PARSAV and PARRES.)

    This command is valid in any processor.  If used in the solution
    processor, this command is valid only within the first load step.
    """
    return self.run(f"RESUME,{fname},{ext},,{nopar},{knoplot}", **kwargs)


def save(self, fname="", ext="", slab="", **kwargs):
    """Saves all current database information.

    APDL Command: SAVE

    Parameters
    ----------
    fname
        File name and directory path (248 characters maximum,
        including the characters needed for the directory path).
        An unspecified directory path defaults to the working
        directory; in this case, you can use all 248 characters
        for the file name.

    ext
        Filename extension (eight-character maximum).

    slab
        Mode for saving the database:

        ALL - Save the model data, solution data and post data
              (element tables, etc.). This value is the default.

        MODEL - Save the model data (solid model, finite element
        model, loadings, etc.) only.

        SOLU - Save the model data and the solution data (nodal
        and element results).

    Notes
    -----
    Saves all current database information to a file (File.DB).
    In interactive mode, an existing File.DB is first written to a
    backup file (File.DBB).  In batch mode, an existing File.DB is
    replaced by the current database information with no backup.
    The command should be issued periodically to ensure a current
    file backup in case of a system "crash" or a "line drop."  It
    may also be issued before a "doubtful" command so that if the
    result is not what was intended the database may be easily
    restored to the previous state.  A save may be time consuming
    for large models.  Repeated use of this command overwrites the
    previous data on the file (but a backup file is first written
    during an interactive run).  When issued from within POST1,
    the nodal boundary conditions in the database (which were read
    from the results file) will overwrite the nodal boundary
    conditions existing on the database file.

    Internal nodes may be created during solution (for example,
    via the mixed u-P formulation or generalized plane strain
    option for current- technology elements, the Lagrangian
    multiplier method for contact elements or the MPC184 elements,
    or the quadratic or cubic option of the BEAM188 and PIPE288
    elements). It is sometimes necessary to save the internal
    nodes in the database for later operations, such as cutting
    boundary interpolations (CBDOF) for submodeling. To do so,
    issue the SAVE command after the first SOLVE command.

    In general, saving after solving is always a good practice.

    This command is valid in any processor.
    """
    return self.run(f"SAVE,{fname},{ext},,{slab}", **kwargs)


def smbc(self, mode="", **kwargs):
    """Controls the display of solid model boundary condition symbols and

    APDL Command: /SMBC
    labels.

    Parameters
    ----------
    mode
        CENT

        CENT - Solid model boundary condition symbols and labels appear at the centroid of the
               solid model entity (default).

        TESS - Solid model boundary condition symbols and labels appear inside each
               constituent element of the tessellation.

    Notes
    -----
    Mode = CENT is designed to reduce the clutter of boundary condition
    symbols in solid model plots. For example, if you have assigned normal
    pressure loads to an area, you may choose to display the pressures as
    arrows with the /PSF command using /PSF,PRES,NORM,2. When Mode = CENT,
    the pressure arrow is displayed at the centroid of the area. When Mode
    = TESS, a pressure arrow is displayed at the centroid of each polygon
    of the area's tessellation.

    This command is valid in any processor.
    """
    command = "/SMBC,%s" % (str(mode))
    return self.run(command, **kwargs)


def stat(self, **kwargs):
    """Displays the status of database settings.

    APDL Command: STAT

    Notes
    -----
    In the DISPLAY program, STAT will show the current status of the
    program settings.

    In the ANSYS program, STAT is a command generated by the GUI and will
    appear in the log file (Jobname.LOG) if status is requested for some
    items under Utility Menu> List> Status.  Generally, STAT will be
    preceded by one of the commands listed below, which specifies the
    particular topic that status was requested for.

    If entered directly into the program, the STAT command should be
    immediately preceded by the desired topic command listed below.  In
    processors other than those listed below (e.g., AUX12), no topic
    command should proceed STAT.

    This command is valid in any processor.

    PREP7 topic commands (and their corresponding topics) are:

    SOLUTION topic commands (and their corresponding topics) are:

    POST1 topic commands (and their corresponding topics) are:

    POST26 topic commands (and their corresponding topics) are:
    """
    command = "STAT,"
    return self.run(command, **kwargs)


def stitle(self, nline="", title="", **kwargs):
    """Defines subtitles.

    APDL Command: /STITLE

    Parameters
    ----------
    nline
        Subtitle line number (1 to 4).  Defaults to 1.

    title
        Input up to 70 alphanumeric characters.  Parameter substitution may
        be forced within the title by enclosing the parameter name or
        parametric expression within percent (%) signs.  If Title is blank,
        this subtitle is deleted.

    Notes
    -----
    Subtitles (4 maximum) are displayed in the output along with the main
    title [/TITLE].  Subtitles do not appear in GUI windows or in ANSYS
    plot displays.  The first subtitle is also written to various ANSYS
    files along with the main title.  Previous subtitles may be overwritten
    or deleted.  Issue /STATUS to display titles.

    This command is valid in any processor.
    """
    command = "/STITLE,%s,%s" % (str(nline), str(title))
    return self.run(command, **kwargs)


def title(self, title="", **kwargs):
    """Defines a main title.

    APDL Command: /TITLE

    Parameters
    ----------
    title
        Input up to 72 alphanumeric characters.  Parameter substitution may
        be forced within the title by enclosing the parameter name or
        parametric expression within percent (%) signs.

    Notes
    -----
    The title is carried through the printout and written on various files.
    The title written to a file is the title defined at that time.  Special
    characters may be used within the title text.  Subtitles may also be
    defined [/STITLE].

    This command is valid in any processor.
    """
    command = "/TITLE,%s" % (str(title))
    return self.run(command, **kwargs)


def units(self, label="", lenfact="", massfact="", timefact="",
          tempfact="", toffset="", chargefact="", forcefact="",
          heatfact="", **kwargs):
    """Annotates the database with the system of units used.

    APDL Command: /UNITS

    Parameters
    ----------
    label
        Label to denote the system of units used in this job:

        USER - User-defined system (default).

        SI - International system (m, kg, s, K).

        MKS - MKS system (m, kg, s, °C).

        uMKS - μMKS system (μm, kg, s, °C).

        CGS - CGS system (cm, g, s, °C).

        MPA - MPA system (mm, Mg, s, °C).

        BFT - U. S. Customary system using feet (ft, slug, s, °F).

        BIN - U. S. Customary system using inches (in, lbf*s2/in, s, °F).

    Notes
    -----
    Allows the user to set a marker in the database indicating the system
    of units used.  The setting may be reviewed with the /STATUS command at
    the Begin level.  The units label and conversion factors on this
    command are for user convenience only and have no effect on the
    analysis or data.  That is, /UNITS will not convert database items from
    one system to another (e.g., from U. S. Customary to SI, etc.).  The
    units setting will be written to the file of IGES data [IGESOUT or
    CDWRITE], which can then be read by many programs that read IGES files.
    The user must still use consistent units for the results to be valid.

    If you choose the MKS system of units, the EPZRO option for the EMUNIT
    command is set to 8.85 e-12 F/m.  (EPZRO specifies alternate free-space
    permittivity.)

    For micro-electromechanical systems (MEMS), where dimensions are on the
    order of microns, see the conversion factors in System of Units in the
    Coupled-Field Analysis Guide.

    If you use the ANSYS ADAMS Interface to export model information to the
    ADAMS program, the /UNITS command is required to ensure the correct
    transfer of data between ANSYS and ADAMS. You may choose a predefined
    unit system label (Label = SI, CGS, etc.) or you can select the user-
    defined system option (Label = USER) and input the appropriate
    conversion factors (LENFACT, MASSFACT, TIMEFACT, and FORCEFACT). The
    conversion factors will be written to the ADAMS input file Jobname.MNF
    in order to correctly generate the load. For more information, see
    Export to ADAMS in the Substructuring Analysis Guide.

    All differences between the base solution units used by the ANSYS and
    CFX solvers will be noted in the ANSYS output file.   Unit conversions
    are automatically applied to all loads transferred unless Label = USER.
    Unit conversions are not applied to any of the loads transferred
    between the ANSYS and CFX solvers if they use a user-defined unit
    system.

    This command is valid in any processor.
    """
    command = "/UNITS,%s,%s,%s,%s,%s,%s,%s,%s,%s" % (str(label), str(lenfact), str(massfact), str(
        timefact), str(tempfact), str(toffset), str(chargefact), str(forcefact), str(heatfact))
    return self.run(command, **kwargs)



"""These SESSION commands control the overall characteristics of the
session, including the jobname, Graphical User Interface behavior, and
file switching.
"""


def config(self, lab="", value="", **kwargs):
    """Assigns values to ANSYS configuration parameters.

    APDL Command: /CONFIG

    Parameters
    ----------
    lab
        Configuration parameter to be changed:

        NORSTGM - Option to write or not write geometry data to the results file. VALUE is either
                  0 (write geometry data) or 1 (do not write geometry
                  data). Useful when complex analyses will create
                  abnormally large files. Default is 0.

        NBUF - VALUE is the number of buffers (1 to 32) per file in the solver.  Defaults to
               4.

        LOCFL - File open and close actions.  For VALUE use: 0 for global (default); 1 for
                local.  Applicable to File.EROT, File.ESAV, and File.EMAT.
                Typically  used for large problems where locally closed
                files may be deleted  earlier in the run with the /FDELE
                command.

        SZBIO - VALUE is the record size (1024 to 4194304) of binary files (in integer words).
                Defaults to 16384 (system dependent).

        ORDER - Automatic reordering scheme.  For VALUE use: 0 for WSORT,ALL; 1 for WAVES; 2
                for both WSORT,ALL and WAVES (default).

        FSPLIT - Defines split points for binary files.  VALUE is the file split point in
                 megawords and defaults to the maximum file size for the
                 system.

        MXND - Maximum number of nodes. If not specified, defaults to 100 at first encounter.
               Dynamically expanded by doubling, even at first encounter,
               when maximum is exceeded.

        MXEL - Maximum number of elements.  Default and expansion as for MXND.

        MXKP - Maximum number of keypoints.  Default and expansion as for MXND.

        MXLS - Maximum number of lines.  Default and expansion as for MXND.

        MXAR - Maximum number of areas.  Default and expansion as for MXND.

        MXVL - Maximum number of volumes.  Default and expansion as for MXND.

        MXRL - Maximum number of sets of real constants (element attributes).  Default and
               expansion as for MXND.

        MXCP - Maximum number of sets of coupled degrees of freedom.  Default and expansion as
               for MXND.

        MXCE - Maximum number of constraint equations.  Default and expansion as for MXND.

        NOELDB - Option to write or not write results into the database after a solution.  When
                 VALUE = 0 (default), write results into the database.
                 When VALUE = 1, do not write results into the database.

        DYNA_DBL - Option to invoke the double precision version of the explicit dynamics solver
                   LS-DYNA. When VALUE = 0 (default), the single precision
                   version is used. When VALUE = 1, the double precision
                   version is used.

        STAT - Displays current values set by the /CONFIG command.

    value
        Value (an integer number) assigned to the configuration parameter.

    Notes
    -----
    All configuration parameters have initial defaults, which in most cases
    do not need to be changed.  Where a specially configured version of the
    ANSYS program is desired, the parameters may be changed with this
    command.  Issue /CONFIG,STAT to display current values.  Changes must
    be defined before the parameter is required.  These changes (and
    others) may also be incorporated into the config162.ans file which is
    read upon execution of the program (see The Configuration File in the
    Basic Analysis Guide).  If the same configuration parameter appears in
    both the  configuration file and this command, this command overrides.

    Distributed ANSYS uses the default FSPLIT value, and forces NOELDB = 1
    and NORSTGM = 0 for all results files. The FSPLIT, NOELDB, and NORSTGM
    options cannot be changed when using Distributed ANSYS.

    The /CONFIG command is not valid for the ANSYS Multiphysics 1, 2, or 3
    products.

    The ANSYS Multi-field solver (MFS and MFX) does not support
    /CONFIG,NOELDB,1. The ANSYS Multi-field solver needs the updated ANSYS
    database.
    """
    command = "/CONFIG,%s,%s" % (str(lab), str(value))
    return self.run(command, **kwargs)


def cwd(self, dirpath="", **kwargs):
    """Changes the current working directory.

    APDL Command: /CWD

    Parameters
    ----------
    dirpath
        The full path name of the new working directory.

    Notes
    -----
     After issuing the /CWD command, all new files opened with no default
    directory specified (via the FILE, /COPY, or RESUME commands, for
    example) default to the new DIRPATH directory.
    """
    command = "/CWD,%s" % (str(dirpath))
    return self.run(command, **kwargs)


def filname(self, fname="", key="", **kwargs):
    """Changes the Jobname for the analysis.

    APDL Command: /FILNAME

    Parameters
    ----------
    fname
        Name (32 characters maximum) to be used as the Jobname.  Defaults
        to the initial Jobname as specified on the ANSYS execution command,
        or to file if none specified.

    key
        Specify whether to use the existing log, error, lock, page, and
        output files (.LOG, .ERR, .LOCK, .PAGE and .OUT) or start new
        files.

        0, OFF - Continue using current log, error, lock, page, and output files.

        1, ON - Start new log, error, lock, page, and output files (old log and error files are
                closed and saved, but old lock, page, and output files are
                deleted). Existing log and error files are appended.

    Notes
    -----
    All subsequently created files will be named with this Jobname if Key =
    0.  Use Key = 1 to start new log, error, lock, page, and output files.
    The previous Jobname is typically defined on the ANSYS program
    execution line (see the Operations Guide).  This command is useful when
    different groups of files created throughout the run are to have
    different names.  For example, the command may be used before each
    substructure pass to avoid overwriting files or having to rename each
    file individually.

    This command is valid only at the Begin level.
    """
    command = "/FILNAME,%s,%s" % (str(fname), str(key))
    return self.run(command, **kwargs)


def input(self, fname="", ext="", dir_="", line="", log="", **kwargs):
    """Switches the input file for the commands that follow.

    APDL Command: /INPUT

    Parameters
    ----------
    fname
        File name and directory path (248 characters maximum, including the
        characters needed for the directory path).  An unspecified
        directory path defaults to the working directory; in this case, you
        can use all 248 characters for the file name.

    ext
        Filename extension (eight-character maximum).

    dir\_
        Directory path (64 characters maximum). Defaults to current
        directory.

    line
        A value indicating either a line number in the file or a user-
        defined label in the file from which to begin reading the input
        file.

        (blank), 0, or 1 - Begins reading from the top of the file (default).

        LINE_NUMBER - Begins reading from the specified line number in the file.

        :label - Begins reading from the first line beginning with the matching user-defined
                 label :label (beginning with a colon (:), 8 characters
                 maximum).

    log
        Indicates whether secondary input from this command should be
        recorded in the command log (File.LOG) and the database log:

        0 - Record only the /INPUT command on the log (default).

        1 - Record commands in the specified secondary file as they are executed.

    Notes
    -----
    Switches the input file for the next commands.  Commands are read from
    this file until an end-of-file or another file switching directive is
    read.  An end-of-file occurs after the last record of the file or when
    a /EOF command is read.  An automatic switch back one level (to the
    previous file) occurs when an end-of-file is encountered.  Twenty
    levels of nested file switching are allowed.  Note that files including
    *DO, *USE, *ULIB, and the "Unknown Command" Macro have less nesting
    available because each of these operations also uses a level of file
    switching.  For an interactive run, a /INPUT,TERM switches to the
    terminal for the next input.  A /EOF read from the terminal then
    switches back to the previous file.  A /INPUT (with a blank second
    field) switches back to the primary input file.

    Setting LOG = 1 on /INPUT causes all commands read from the specified
    file to be recorded in the command log (File.LOG) and the internal
    database command log [LGWRITE].  This option is recommended if the log
    file will be used later .  The LOG = 1 option is only valid when the
    /INPUT occurs in the primary input file.  Using LOG = 1 on a nested
    /INPUT or on a /INPUT within a do-loop will have no effect (i.e.,
    commands in the secondary input file are not written to the command
    log).

    The Dir option is optional as the directory path can be included
    directly in Fname.

    This command is valid in any processor.
    """
    command = "/INPUT,%s,%s,%s,%s,%s" % (str(fname),
                                         str(ext), str(dir_), str(line), str(log))
    return self.run(command, **kwargs)


def keyw(self, keyword="", key="", **kwargs):
    """Sets a keyword used by the GUI for context filtering (GUI).

    APDL Command: KEYW

    Parameters
    ----------
    keyword
        A keyword which, when set to either true or false, changes the
        behavior of the GUI.

    key
        Keyword switch:

        0 - Sets the keyword to "false."

        1 - Sets the keyword to "true."

    Notes
    -----
    Defines a keyword used by the GUI for context filtering.  This is a
    command generated by the GUI and may appear in the log file
    (Jobname.LOG) if the GUI is used.  This command is usually not typed in
    directly in an ANSYS session.

    This command is valid in any processor.
    """
    command = "KEYW,%s,%s" % (str(keyword), str(key))
    return self.run(command, **kwargs)


def memm(self, lab="", kywrd="", **kwargs):
    """Allows the current session to keep allocated memory.

    APDL Command: MEMM

    Parameters
    ----------
    lab
        When Lab = KEEP, the memory manager's ability to acquire and keep
        memory is controlled by Kywrd

    kywrd
        Turns the memory "keep" mode on or off

        ON - Keep any memory allocated during the analysis.

        OFF - Use memory dynamically and free it up to other users after use (default).

    Notes
    -----
    You can use the MEMM command to ensure that memory intensive operations
    will always have the same memory available when the operations occur
    intermittently. Normally, if a large amount of memory is allocated for
    a specific operation, it will be returned to the system once the
    operation is finished. This option always maintains the highest level
    used during the analysis until the analysis is finished.

    The MEMM command does not affect the value you specify with the -m
    switch. When you allocate memory with the -m switch, that amount will
    always be available. However, if dynamic memory allocation in excess of
    the-m value occurs, you can use the MEMM command to ensure that amount
    is retained until the end of your analysis.
    """
    command = "MEMM,%s,%s" % (str(lab), str(kywrd))
    return self.run(command, **kwargs)


def nerr(self, nmerr="", nmabt="", abort="", ifkey="", num="", **kwargs):
    """Limits the number of warning and error messages displayed.

    APDL Command: /NERR

    Parameters
    ----------
    nmerr
        Maximum number of warning and error messages displayed per
        command.  Defaults to 5 for interactive runs with the GUI
        turned on, 20 for interactive runs with the GUI turned
        off, 200 for batch runs.  If NMERR is negative, the
        absolute value of NMERR is used as the maximum number of
        warning and error messages written to the error file
        (file.ERR) per command, as well as the maximum number of
        messages displayed per command.

    nmabt
        Maximum number of warning and error messages allowed per
        command before run aborts (must be greater than zero).
        Maximum value is 99,999,999. Defaults to 10,000.

    abort
        Abort level key.  Set to 0 for default abort behavior, -1
        to never abort, and -2 to abort after ``nmabt`` errors.
        Altering the abort level key is not recommended, but can
        be helpful for avoiding an abort within /BATCH mode but
        using ``pyansys`` interactively.

    ifkey
        Specifies whether or not to abort if an error occurs during a
        /INPUT operation:

        0 or OFF - Do not abort. This option is the default.

        1 or ON - Abort.

    num
        The number of invalid command warnings before a stop warning will
        be issued:

        0 - Disables the stop warning/error function.

        n - An integer value representing the number of warnings that will be encountered
            before prompting the user to stop (default = 5). The first
            error encountered will ALWAYS result in a prompt.

    Notes
    -----
    Limits the number of warning and error messages displayed for any one
    command in an interactive run.

    Warning and error messages continue to be written to Jobname.ERR
    regardless of these limits (unless NMERR is negative).

    Issue this command with NUM = n to specify the number of "invalid
    command" warnings to be encountered before the user is prompted to
    stop. You can then continue or abort the run. If you choose to abort
    the run, the log file can be saved so that any of the processing up to
    that point can be appended to an input that rectifies the condition. A
    batch run always aborts on the first error.  Issue /NERR,STAT to list
    current settings.

    Issue /NERR,DEFA to reset values to initial defaults.

    An IFKEY value of 1 or ON causes the ANSYS program to abort immediately
    upon encountering an error during a file /INPUT operation. However, use
    of this option may cause the following conditions to occur:

    The /INPUT command may abort if issued for a log file (jobname.log).

    Some macros may abort.

    A CAD connection may fail after reading only a small portion of a CAD
    model.

    The command is valid in any processor.
    """
    command = "/NERR,%s,%s,%s,%s,%s" % (str(nmerr), str(nmabt), str(abort),
                                        str(ifkey), str(num))
    return self.run(command, **kwargs)


def pause(self, **kwargs):
    """Temporarily releases the current product license.

    APDL Command: PAUSE

    Notes
    -----
    The PAUSE command temporarily releases (or pauses) the current product
    license so that another application can use it.

    This application consumes a license as soon as you launch it, and
    retains that license until it is finished. If you launch the product
    interactively, the license is retained until you either close the
    application or issue a PAUSE command via the command line.

    No other operation (other than SAVE or /EXIT) is possible in the
    current application while use of the product license is paused.

    When the second application has finished and releases the license,
    issue an UNPAUSE command via the command line to restore use of the
    license to the current application.

    For more information, see the ANSYS, Inc. Licensing Guide.
    """
    command = "PAUSE,"
    return self.run(command, **kwargs)


def slashstatus(self, lab="", **kwargs):
    """Lists the status of items for the run.

    APDL Command: /STATUS

    Parameters
    ----------
    lab
        Items to list status for:

        ALL - List all below (default).

        TITLE - List only titles, Jobname, and revision number.

        UNITS - List only units.

        MEM - List only memory data statistics.

        DB - List only database statistics

        CONFIG - List only configuration parameters.

        GLOBAL - Provides a global status summary.

        SOLU - Provides a solution status summary.

        PROD - Provides a product summary.

    Notes
    -----
    Displays various items active for the run (such as the ANSYS revision
    number, Jobname, titles, units, configuration parameters, database
    statistics, etc.).

    This command is valid in any processor.
    """
    return self.run(f"/STATUS,{lab}", **kwargs)


def starstatus(self, par="", imin="", imax="", jmin="", jmax="", kmin="",
               kmax="", lmin="", lmax="", mmin="", mmax="", kpri="",
               **kwargs):
    """Lists the current parameters and abbreviations.

    APDL Command: *STATUS

    Parameters
    ----------
    par
        Specifies the parameter or sets of parameters listed. For array
        parameters, use IMIN, IMAX, etc. to specify ranges.  Use *DIM to
        define array parameters. Use *VEDIT to review array parameters
        interactively. Use *VWRITE to print array values in a formatted
        output. If Par is blank, list all scalar parameter values, array
        parameter dimensions, and abbreviations.  If ARGX, list the active
        set of local macro parameters (ARG1 to ARG9 and AR10 to AR99)
        [*USE].

        Lists all parameters (except local macro parameters and those with names beginning or ending with an underbar) and toolbar abbreviations. - Lists only parameters with names beginning with an underbar (_). These are
                          ANSYS internal parameters.

        Lists only parameters with names ending with an underbar (_). A good APDL programming convention is to ensure that all parameters created by your system programmer are named with a trailing underbar. - Lists all toolbar abbreviations.

        Lists all parameters (except local macro parameters and those with names beginning or ending with an underbar). - Lists all APDL Math parameters, including vectors, matrices, and linear
                          solvers.

        Lists only the parameter specified. PARNAME cannot be a local macro parameter name. - Lists all local macro parameter values (ARG1- AR99) that are non-zero or non-
                          blank.

    imin, imax, jmin, jmax, kmin, kmax, lmin, lmax, mmin, mmax
        Range of array elements to display (in terms of the dimensions
        (row, column, plane, book, and shelf).   Minimum values default to
        1.  Maximum values default to the maximum dimension values.  Zero
        may be input for IMIN, JMIN, and KMIN to display the index numbers.
        See *TAXIS  command to list index numbers of 4- and 5-D tables.

    kpri
        Use this field to list your primary variable labels (X, Y, Z, TIME,
        etc.).

        List the labels (default). YES, Y, or ON are also valid entries.  - Do not list the labels. NO, N, or OFF are also valid entries.

    Notes
    -----
    You cannot obtain the value for a single local parameter (e.g.,
    *STATUS,ARG2). You can only request all local parameters simultaneously
    using *STATUS,ARGX.

    This command is valid in any processor.
    """
    command = "*STATUS,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s" % (str(par), str(imin), str(imax), str(
        jmin), str(jmax), str(kmin), str(kmax), str(lmin), str(lmax), str(mmin), str(mmax), str(kpri))
    return self.run(command, **kwargs)


def syp(self, string="", arg1="", arg2="", arg3="", arg4="", arg5="",
        arg6="", arg7="", arg8="", **kwargs):
    """Passes a command string and arguments to the operating system.

    APDL Command: /SYP

    Parameters
    ----------
    string
        Command string (cannot include commas).  See also the /SYS command.

    arg1, arg2, arg3, . . . , arg8
        Arguments to be appended to the command string, separated by
        blanks, commas, or other delimiter characters (see the Operations
        Guide).  The arguments may be numbers, parameters, or parametric
        expressions.

    Notes
    -----
    Passes a command string to the operating system for execution, along
    with arguments to be appended to the command string.  See the
    Operations Guide for details.  ANSYS may not be aware of your specific
    user environment. For example, on Linux this command may not recognize
    aliases, depending on the hardware platform and user environment.

    This command is valid in any processor.
    """
    command = "/SYP,%s,%s,%s,%s,%s,%s,%s,%s,%s" % (str(string), str(arg1), str(
        arg2), str(arg3), str(arg4), str(arg5), str(arg6), str(arg7), str(arg8))
    return self.run(command, **kwargs)


def sys(self, string="", **kwargs):
    """Passes a command string to the operating system.

    APDL Command: /SYS

    Parameters
    ----------
    string
        Command string, up to 639 characters (including blanks, commas,
        etc.). The specified string is passed verbatim to the operating
        system, i.e., no parameter substitution is performed.

    Notes
    -----
    Passes a command string to the operating system for execution (see the
    Operations Guide).  Typical strings are system commands such as list,
    copy, rename, etc.  Control returns to the ANSYS program after the
    system procedure is completed.   ANSYS may not be aware of your
    specific user environment. For example, on Linux this command may not
    recognize aliases, depending on the hardware platform and user
    environment.

    This command is valid in any processor.
    """
    command = "/SYS,%s" % (str(string))
    return self.run(command, **kwargs)


def unpause(self, **kwargs):
    """Restores use of a temporarily released product license.

    APDL Command: UNPAUSE

    Notes
    -----
    The UNPAUSE command restores use of a temporarily released (paused)
    product license. The command is valid only after a previously issued
    PAUSE command.

    When use of the product license is paused via the PAUSE command, no
    other operation (other than SAVE or /EXIT) is possible until you issue
    the UNPAUSE command.

    For more information, see the documentation for the PAUSE command and
    the ANSYS, Inc. Licensing Guide.
    """
    command = "UNPAUSE,"
    return self.run(command, **kwargs)



"""These commands may be run by PyMapdl, but are not supported from a
user-context.

"""


def batch(self, lab="", **kwargs):
    """APDL Command: /BATCH

    Sets the program mode to "batch."

    Parameters
    ----------
    lab
        Specifies listing mode during a batch run:

        LIST - The batch output will include a listing of the input file.

        (blank) - Suppress input data listing.

    Notes
    -----
    Sets the program mode to "batch" when included as the first line on an
    input file of ANSYS commands.  For convenience, this command is
    automatically recorded on the log file (Jobname.LOG) at the beginning
    of an interactive ANSYS session so that the log file can be reused
    later for batch input.

    Caution:: : This command  should not be entered directly in an
    interactive ANSYS session since all subsequent keyboard input is simply
    copied to a file, without further processing or prompts from the
    program (use the "system break" to exit the ANSYS program if this
    occurs).

    The recommended method for choosing batch mode, rather than using the
    /BATCH command, is to select the Batch simulation environment from the
    ANSYS Product Launcher  task in the ANSYS launcher, or the batch mode
    entry option on the ANSYS execution command when entering the program.

    This command is valid only at the Begin Level.
    """
    command = "/BATCH,%s" % (str(lab))
    return self.run(command, **kwargs)


def output(self, fname="", ext="", loc="", **kwargs):
    """Redirects text output to a file or to the screen.

    APDL Command: /OUTPUT

    Parameters
    ----------
    fname
        Filename and directory path (248 character maximum, including
        directory) to which text output will be redirected (defaults to
        Jobname if Ext is specified).  For interactive runs, Fname = TERM
        (or blank) redirects output to the screen.  For batch runs, Fname =
        blank (with all remaining command arguments blank) redirects output
        to the  default system output file.

    ext
        Filename extension (eight-character maximum).

    loc
        Location within a file to which output will be written:

        (blank) - Output is written starting at the top of the file (default).

        APPEND - Output is appended to the existing file.

    Notes
    -----
    Text output includes responses to every command and GUI
    function, notes, warnings, errors, and other informational
    messages.  Upon execution of /OUTPUT,Fname, Ext, ..., all
    subsequent text output is redirected to the file Fname.Ext.
    To redirect output back to the default location, issue /OUTPUT
    (no arguments).

    This command is valid in any processor.
    """
    if loc:
        return self.run(f"/OUTPUT,{fname},{ext},,{loc}", **kwargs)
    else:
        return self.run(f"/OUTPUT,{fname},{ext}", **kwargs)



def mfouri(self, oper="", coeff="", mode="", isym="", theta="", curve="",
           **kwargs):
    """Calculates the coefficients for, or evaluates, a Fourier series.

    APDL Command: *MFOURI

    Parameters
    ----------
    oper
        Type of Fourier operation:

        Calculate Fourier coefficients COEFF from MODE, ISYM, THETA, and CURVE. - Evaluate the Fourier curve CURVE from COEFF, MODE, ISYM andTHETA

    coeff
        Name of the array parameter vector containing the Fourier
        coefficients (calculated if Oper = FIT, required as input if Oper =
        EVAL).  See *SET for name restrictions.

    mode
        Name of the array parameter vector containing the mode numbers of
        the desired Fourier terms.

    isym
        Name of the array parameter vector containing the symmetry key for
        the corresponding Fourier terms.  The vector should contain keys
        for each term as follows:

        Symmetric (cosine) term - Antisymmetric (sine) term.

    theta, curve
        Names of the array parameter vectors containing the theta vs. curve
        description, respectively.  Theta values should be input in
        degrees.  If Oper = FIT, one curve value should be supplied with
        each theta value.  If Oper = EVAL, one curve value will be
        calculated for each theta value.

    Notes
    -----
    Calculates the coefficients of a Fourier series for a given curve, or
    evaluates the Fourier curve from the given (or previously calculated)
    coefficients.  The lengths of the COEFF, MODE, and ISYM vectors must be
    the same--typically two times the number of modes desired, since two
    terms (sine and cosine) are generally required for each mode.  The
    lengths of the CURVE and THETA vectors should be the same or the
    smaller of the two will be used.  There should be a sufficient number
    of points to adequately define the curve--at least two times the number
    of coefficients.  A starting array element number (1) must be defined
    for each array parameter vector.  The vector specifications *VLEN,
    *VCOL, *VABS, *VFACT, and *VCUM do not apply to this command.  Array
    elements should not be skipped with the *VMASK and the NINC value of
    the *VLEN specifications.  The vector being calculated (COEFF if Oper
    is FIT, or CURVE if Oper is EVAL) must exist as a dimensioned array
    [*DIM].

    This command is valid in any processor.
    """
    command = f"*MFOURI,{oper},{coeff},{mode},{isym},{theta},{curve}"
    return self.run(command, **kwargs)


def mfun(self, parr="", func="", par1="", **kwargs):
    """Copies or transposes an array parameter matrix.

    APDL Command: *MFUN

    Parameters
    ----------
    parr
        The name of the resulting array parameter matrix.  See *SET for
        name restrictions.

    func
        Copy or transpose function:

        Par1 is copied to ParR - Par1 is transposed to ParR.  Rows (m) and columns (n) of Par1 matrix are
                          transposed to resulting ParR matrix of shape
                          (n,m).

    par1
        Array parameter matrix input to the operation.

    Notes
    -----
    Operates on one input array parameter matrix and produces one output
    array parameter matrix according to:

    ParR = f(Par1)

    where the function (f) is either a copy or transpose, as described
    above.

    Functions are based on the standard FORTRAN definitions where possible.
    ParR may be the same as Par1.  Starting array element numbers must be
    defined for each array parameter matrix if it does not start at the
    first location. For example, *MFUN,A(1,5),COPY,B(2,3) copies matrix B
    (starting at element (2,3)) to matrix A (starting at element (1,5)).
    The diagonal corner elements for each submatrix must be defined: the
    upper left corner by the array starting element (on this command), the
    lower right corner by the current values from the *VCOL and *VLEN
    commands.  The default values are the (1,1) element and the last
    element in the matrix.  No operations progress across matrix planes (in
    the 3rd dimension).  Absolute values and scale factors may be applied
    to all parameters [*VABS, *VFACT].  Results may be cumulative [*VCUM].
    Array elements should not be skipped with the *VMASK and the NINC value
    of the *VLEN specifications.  The number of rows [*VLEN] applies to the
    Par1 array.  See the *VOPER command for details.

    This command is valid in any processor.
    """
    command = f"*MFUN,{parr},{func},{par1}"
    return self.run(command, **kwargs)


def moper(self, parr="", par1="", oper="", val1="", val2="",
          val3="", val4="", val5="", val6="", **kwargs):
    """ Performs matrix operations on array parameter matrices.

    APDL Command: *MOPER

    Parameters
    ----------
    parr
        The name of the resulting array parameter matrix.  See *SET for
        name restrictions.

    par1
        First array parameter matrix input to the operation. For Oper =
        MAP, this is an N x 3 array of coordinate locations at which to
        interpolate. ParR will then be an N(out) x M array containing the
        interpolated values.

    oper
        Matrix operations:

        INVERT - (*MOPER, ParR, Par1, INVERT)
        Square matrix invert: Inverts the n x n matrix in Par1
        into ParR. The matrix must be well conditioned.

        Warning: Non-independent or ill-conditioned equations can
        cause erroneous results. - For large matrices, use the
        APDL Math operation *LSFACTOR for efficiency (see APDL
        Math).

        MULT - (*MOPER, ParR, Par1, MULT, Par2)
        Matrix multiply: Multiplies Par1 by Par2.  The number of
        rows of Par2 must equal the number of columns of Par1 for
        the operation. If Par2 is input with a number of rows
        greater than the number of columns of Par1, matrices are
        still multiplied. However, the operation only uses a
        number of rows of Par2 equal to the number of columns of
        Par1.

        COVAR - (*MOPER, ParR, Par1, COVAR, Par2)
        Covariance: The measure of association between two columns
        of the input matrix (Par1).  Par1, of size m runs (rows)
        by n data (columns) is first processed to produce a row
        vector containing the mean of each column which is
        transposed to a column vector (Par2) of n array elements.
        The Par1 and Par2 operation then produces a resulting n x
        n matrix (ParR) of covariances (with the variances as the
        diagonal terms).

        CORR - (*MOPER, ParR, Par1, CORR, Par2)
        Correlation: The correlation coefficient between two
        variables.  The input matrix (Par1), of size m runs (rows)
        by n data (columns), is first processed to produce a row
        vector containing the mean of each column which is then
        transposed to a column vector (Par2) of n array elements.
        The Par1 and Par2 operation then produces a resulting n x
        n matrix (ParR) of correlation coefficients (with a value
        of 1.0 for the diagonal terms).

        SOLV - (*MOPER, ParR, Par1, SOLV, Par2)
        Solution of simultaneous equations: Solves the set of n
        equations of n terms of the form an1x1 + an2x2 + ... +
        annxn = bn where Par1 contains the matrix of
        a-coefficients, Par2 the vector(s) of b-values, and ParR
        the vector(s) of x-results.  Par1 must be a square matrix.
        The equations must be linear, independent, and well
        conditioned.

        Warning: Non-independent or ill-conditioned equations can
        cause erroneous results. - For large matrices, use the
        APDL Math operation *LSFACTOR for efficiency (see APDL
        Math).

        SORT - (*MOPER, ParR, Par1, SORT, Par2, n1, n2, n3)
        Matrix sort: Sorts matrix Par1 according to sort vector
        Par2 and places the result back in Par1. Rows of Par1 are
        moved to the corresponding positions indicated by the
        values of Par2. Par2 may be a column of Par1 (in which
        case it will also be reordered). Alternatively, you may
        specify the column of Par1 to sort using n1 (leaving Par2
        blank). A secondary sort can be specified by column n2,
        and a third sort using n3. ParR is the vector of initial
        row positions (the permutation vector).  Sorting Par1
        according to ParR should reproduce the initial ordering.

        NNEAR - (*MOPER, ParR, Par1, NNEAR, Toler)
        Nearest Node: Quickly determine all the nodes within a
        specified tolerance of a given array.  ParR is a vector of
        the nearest selected nodes, or 0 if no nodes are nearer
        than Toler. Par1 is the n x 3 array of coordinate
        locations. Toler defaults to 1 and is limited to the
        maximum model size. - (*MOPER, ParR, Par1, ENEAR, Toler)

        ENEAR - (*MOPER, ParR, Par1, ENEAR, Toler)
        Nearest Element: Quickly determine the elements with
        centroids that are within a specified tolerance of the
        points in a given array. - ParR is a vector of the nearest
        selected elements, or 0 if no element centroids are nearer
        than Toler. Par1 is the n x 3 array of coordinate
        locations.

        MAP - (*MOPER, ParR, Par1, MAP, Par2, Par3, kDim, --, kOut, LIMIT)

        Maps the results from one set of points to another. For
        example, you can map pressures from a CFD analysis onto
        your model for a structural analysis.

        Par1 is the Nout x 3 array of points that will be mapped
        to. Par2 is the Nin x M array that contains M values of
        data to be interpolated at each point and corresponds to
        the Nin x 3 points in Par3. The resulting ParR is the Nout
        x M array of mapped data points.

        For each point in the destination mesh, all possible
        triangles in the source mesh are searched to find the best
        triangle containing each point. It then does a linear
        interpolation inside this triangle. You should carefully
        specify your interpolation method and search criteria in
        order to provide faster and more accurate results (see
        LIMIT, below).

        kDim is the interpolation criteria. If kDim = 2 or 0, two
        dimensional interpolation is applied (interpolate on a
        surface). If kDim = 3, three dimensional interpolation is
        applied (interpolate on a volume).

        kOut specified how points outside of the domain are
        handled. If kOut = 0, use the value(s) of the nearest
        region point for points outside of the region. If kOut =
        1, set results outside of the region to zero.

        LIMIT specifies the number of nearby points considered for
        interpolation. The default is 20, and the minimum is
        5. Lower values will reduce processing time; however, some
        distorted or irregular sets of points will require a
        higher LIMIT value to encounter three nodes for
        triangulation.

        Output points are incorrect if they are not within the
        domain (area or volume) defined by the specified input
        points. Also, calculations for out-of-bound points require
        much more processing time than do points that are within
        bounds. Results mapping is available from the command line
        only.

        INTP - (*MOPER, ParR, Par1, INTP, Par2)
        Finds the elements that contain each point in the array of
        n x 3 points in Par1. Par2 will contain the set of element
        ID numbers and ParR will contain their n x 3 set of
        natural element coordinates (values between -1 and
        1). Par1 must be in global Cartesian coordinates.

        SGET - (*MOPER, ParR, Par1, SGET, Par2, Label, Comp)
        Gets the nodal solution item corresponding to Label and
        Comp (see the PLNSOL command) and interpolates it to the
        given element locations. Par1 contains the n x 3 array of
        natural element coordinates (values between -1 and 1) of
        the n element ID numbers in Par2. Par1 and Par2 are
        usually the output of the *MOPER,,,INTP operation. ParR
        contains the n interpolated results.

        Val1, Val2, ..., Val6
        Additional input used in the operation. The meanings of
        Val1 through Val6 vary depending on the specified matrix
        operation. See the description of Oper for details.
    """
    command = f"*MOPER,{parr},{par1},{oper},{val1},{val2},{val3},{val4},{val5},{val6}"
    return self.run(command, **kwargs)


def mwrite(self, parr="", fname="", ext="", label="", n1="", n2="", n3="",
           **kwargs):
    """Writes a matrix to a file in a formatted sequence.

    APDL Command: *MWRITE

    Parameters
    ----------
    parr
        The name of the array parameter. See *SET for name restrictions.

    fname
        File name and directory path (248 characters maximum, including the
        characters needed for the directory path).  An unspecified
        directory path defaults to the working directory; in this case, you
        can use all 248 characters for the file name.

    ext
        Filename extension (eight-character maximum).

    label
        Can use a value of IJK, IKJ, JIK, JKI, KIJ, KJI, or blank (JIK).

    n1, n2, n3
        Write as (((ParR(i,j,k), k = 1,n1), i = 1, n2), j = 1, n3) for
        Label = KIJ. n1, n2, and n3 default to the corresponding dimensions
        of the array parameter ParR.

    Notes
    -----
    Writes a matrix or vector to a specified file in a formatted sequence.
    You can also use the *VWRITE command to write data to a specified file.
    Both commands contain format descriptors on the line immediately
    following the command. The format descriptors can be in either Fortran
    or C format.

    Fortran format descriptors are enclosed in parentheses. They must
    immediately follow the *MWRITE command on a separate line of the same
    input file. The word FORMAT should not be included. The format must
    specify the number of fields to be written per line, the field width,
    the placement of the decimal point, etc. There should be one field
    descriptor for each data item written. The write operation uses the
    available system FORTRAN FORMAT conventions (see your system FORTRAN
    manual). Any standard FORTRAN real format (such as (4F6.0),
    (E10.3,2X,D8.2), etc.) and character format (A) may be used.  Integer
    (I) and list-directed (*) descriptors may not be used. Text may be
    included in the format as a quoted string. The FORTRAN descriptor must
    be enclosed in parentheses and the format must not exceed 80 characters
    (including parentheses).

    The "C" format descriptors are used if the first character of the
    format descriptor line is not a left parenthesis. "C" format
    descriptors may be up to 80 characters long, consisting of text strings
    and predefined "data descriptors" between the strings where numeric or
    alphanumeric character data are to be inserted. The normal descriptors
    are %I for integer data, %G for double precision data, %C for
    alphanumeric character data, and %/ for a line break. There must be one
    data descriptor for each specified value in the order of the specified
    values. The enhanced formats described in *MSG may also be used.

    The starting array element number must be defined. Looping continues in
    the directions indicated by the Label argument. The number of loops and
    loop skipping may also be controlled with the *VLEN and *VMASK
    commands, which work in the n2 direction (by row on the output file),
    and by  the *VCOL command, which works in the n1 direction (by column
    in the output file).  The vector specifications *VABS and *VFACT apply
    to this command, while *VCUM does not apply to this command. See the
    *VOPER command for details. If you are in the GUI, the *MWRITE command
    must be contained in an externally prepared file and read into ANSYS
    (i.e., *USE, /INPUT, etc.).

    This command is valid in any processor.
    """
    command = f"*MWRITE,{parr},{fname},{ext},,{label},{n1},{n2},{n3}"
    return self.run(command, **kwargs)


def starvput(self, parr="", entity="", entnum="", item1="", it1num="",
             item2="", it2num="", kloop="", **kwargs):
    """Restores array parameter values into the ANSYS database.

    APDL Command: *VPUT

    Parameters
    ----------
    parr
        The name of the input vector array parameter.  See *SET for name
        restrictions.  The parameter must exist as a dimensioned array
        [*DIM] with data input.

    entity
        Entity keyword.  Valid keywords are shown for Entity = in the table
        below.

    entnum
        The number of the entity (as shown for ENTNUM= in the table below).

    item1
        The name of a particular item for the given entity.  Valid items
        are as shown in the Item1 columns of the table below.

    it1num
        The number (or label) for the specified Item1 (if any).  Valid
        IT1NUM values are as shown in the IT1NUM columns of the table
        below.  Some Item1 labels do not require an IT1NUM value.

    item2, it2num
        A second set of item labels and numbers to further qualify the item
        for which data is to be stored.  Most items do not require this
        level of information.

    kloop
        Field to be looped on:

        Loop on the ENTNUM field (default). - Loop on the Item1 field.

        Loop on the IT1NUM field.  Successive items are as shown with IT1NUM. - Loop on the Item2 field.

    Notes
    -----
    The *VPUT command is not supported for PowerGraphics displays.
    Inconsistent results may be obtained if this command is not used in
    /GRAPHICS, FULL.

    Plot and print operations entered via the GUI (Utility Menu> Pltcrtls,
    Utility Menu> Plot) incorporate the AVPRIN command. This means that the
    principal and equivalent values are recalculated. If you use *VPUT to
    put data back into the database, issue the plot commands from the
    command line to preserve your data.

    This operation is basically the inverse of the *VGET operation.  Vector
    items are put directly (without any coordinate system transformation)
    into the ANSYS database.  Items can only replace existing items of the
    database and not create new items.  Degree of freedom results that are
    replaced in the database are available for all subsequent
    postprocessing operations.  Other results are changed temporarily and
    are available mainly for the immediately following print and display
    operations.  The vector specification *VCUM does not apply to this
    command.  The valid labels for the location fields (Entity, ENTNUM,
    Item1, and IT1NUM) are listed below.  Item2 and IT2NUM are not
    currently used.  Not all items from the *VGET list are allowed on *VPUT
    since putting values into some locations could cause the database to be
    inconsistent.

    This command is valid in any processor.

    Table: 250:: : *VPUT - POST1 Items

    X, Y, or Z fluid velocity. X, Y, or Z nodal velocity in a transient
    structural analysis (LS-DYNA analysis or analysis with ANTYPE,TRANS).

    X, Y, or Z magnetic vector potential. X, Y, or Z nodal acceleration in
    a transient structural analysis (LS-DYNA analysis or analysis with
    ANTYPE,TRANS).
    """
    command = f"*VPUT,{parr},{entity},{entnum},{item1},{it1num},{item2},{it2num},{kloop}"
    return self.run(command, **kwargs)


def sread(self, strarray="", fname="", ext="", nchar="", nskip="",
          nread="", **kwargs):
    """Reads a file into a string array parameter.

    APDL Command: *SREAD

    Parameters
    ----------
    strarray
        Name of the "string array" parameter which will hold the read file.
        String array parameters are similar to character arrays, but each
        array element can be as long as 128 characters. If the string
        parameter does not exist, it will be created. The array will be
        created as: *DIM,StrArray,STRING,nChar,nRead

    fname
        File name and directory path (248 characters maximum, including the
        characters needed for the directory path).  An unspecified
        directory path defaults to the working directory; in this case, you
        can use all 248 characters for the file name.

    ext
        Filename extension (eight-character maximum).

    nchar
        Number of characters per line to read (default is length of the
        longest line in the file).

    nskip
        Number of lines to skip at the start of the file (default is 0).

    nread
        Number of lines to read from the file (default is the entire file).

    Notes
    -----
    The *SREAD command reads from a file into a string array
    parameter. The file must be an ASCII text file.
    """
    command = f"*SREAD,{strarray},{fname},{ext},,{nchar},{nskip},{nread}"
    return self.run(command, **kwargs)


def toper(self, parr="", par1="", oper="", par2="", fact1="", fact2="",
          con1="", **kwargs):
    """Operates on table parameters.

    APDL Command: *TOPER

    Parameters
    ----------
    parr
        Name of the resulting table parameter. The command will create a
        table array parameter with this name.  Any existing parameter with
        this name will be overwritten.

    par1
        Name of the first table parameter.

    oper
        The operation to be performed: ADD.  The operation is:  ParR(i,j,k)
        =   FACT1*Par1(i,j,k) + FACT2 *Par2(i,j,k) +CON1

    par2
        Name of the second table parameter.

    fact1
        The first table parameter multiplying constant. Defaults to 1.

    fact2
        The second table parameter multiplying constant.  Defaults to 1.

    con1
        The constant increment for offset.  Defaults to 0.

    Notes
    -----
    *TOPER operates on table parameters according to: ParR(i,j,k) =
    FACT1*Par1(i,j,k) + FACT2 *Par2(i,j,k) +CON1

    Par1 and Par2 must have the same dimensions and the same variable names
    corresponding to those dimensions. Par1 and Par2 must also have
    identical index values for rows, columns, etc.

    If you want a local coordinate system for the resulting array, you must
    dimension it as such using the *DIM command before issuing *TOPER.

    This command is valid in any processor.
    """
    command = f"*TOPER,{parr},{par1},{oper},{par2},{fact1},{fact2},{con1}"
    return self.run(command, **kwargs)


def vabs(self, kabsr="", kabs1="", kabs2="", kabs3="", **kwargs):
    """Applies the absolute value function to array parameters.

    APDL Command: *VABS

    Parameters
    ----------
    kabsr
        Absolute value of results parameter:

        Do not take absolute value of results parameter (ParR). - Take absolute value.

    kabs1
        Absolute value of first parameter:

        Do not take absolute value of first parameter (Par1 or ParI). - Take absolute value.

    kabs2
        Absolute value of second parameter:

        Do not take absolute value of second parameter (Par2 or ParJ). - Take absolute value.

    kabs3
        Absolute value of third parameter:

        Do not take absolute value of third parameter (Par3 or ParK). - Take absolute value.

    Notes
    -----
    Applies an absolute value to parameters used in certain  *VXX and  *MXX
    operations.  Typical absolute value applications are of the form:

    ParR = |f(|Par1|)|

    or

    ParR = |(|Par1| o |Par2|)|

    The absolute values are applied to each input parameter value before
    the operation and to the result value after the operation.  Absolute
    values are applied before the scale factors so that negative scale
    factors may be used.  The absolute value settings are reset to the
    default (no absolute value) after each *VXX or *MXX operation.  Use
    *VSTAT to list settings.

    This command is valid in any processor.
    """
    command = f"*VABS,{kabsr},{kabs1},{kabs2},{kabs3}"
    return self.run(command, **kwargs)


def vcol(self, ncol1="", ncol2="", **kwargs):
    """Specifies the number of columns in matrix operations.

    APDL Command: *VCOL

    Parameters
    ----------
    ncol1
        Number of columns to be used for Par1 with *MXX operations.
        Defaults to whatever is needed to fill the result array.

    ncol2
        Number of columns to be used for Par2 with *MXX operations.
        Defaults to whatever is needed to fill the result array.

    Notes
    -----
    Specifies the number of columns to be used in array parameter matrix
    operations.  The size of the submatrix used is determined from the
    upper left starting array element (defined on the operation command) to
    the lower right array element (defined by the number of columns on this
    command and the number of rows on the *VLEN command).

    The default NCOL is calculated from the maximum number of columns of
    the result array (the *DIM column dimension) minus the starting
    location + 1.  For example, *DIM,R,,1,10 and a starting location of
    R(1,7) gives a default of 4 columns ( starting with R(1,7), R(1,8),
    R(1,9), and R(1,10)).  Repeat operations automatically terminate at the
    last column of the result array.  Existing values in the rows and
    columns of the results matrix remain unchanged where not overwritten by
    the requested input or operation values.

    The column control settings are reset to the defaults after each *MXX
    operation.  Use *VSTAT to list settings.

    This command is valid in any processor.
    """
    command = f"*VCOL,{ncol1},{ncol2}"
    return self.run(command, **kwargs)


def vcum(self, key="", **kwargs):
    """Allows array parameter results to add to existing results.

    APDL Command: *VCUM

    Parameters
    ----------
    key
        Accumulation key:

        Overwrite results. - Add results to the current value of the results parameter.

    Notes
    -----
    Allows results from certain *VXX and *MXX operations to overwrite or
    add to existing results.  The cumulative operation is of the form:

    ParR = ParR + ParR(Previous)

    The cumulative setting is reset to the default (overwrite) after each
    *VXX or *MXX operation.  Use *VSTAT to list settings.

    This command is valid in any processor.
    """
    command = f"*VCUM,{key}"
    return self.run(command, **kwargs)


def vfact(self, factr="", fact1="", fact2="", fact3="", **kwargs):
    """Applies a scale factor to array parameters.

    APDL Command: *VFACT

    Parameters
    ----------
    factr
        Scale factor applied to results (ParR) parameter.  Defaults to 1.0.

    fact1
        Scale factor applied to first parameter (Par1 or ParI).  Defaults
        to 1.0.

    fact2
        Scale factor applied to second parameter (Par2 or ParJ).  Defaults
        to 1.0.

    fact3
        Scale factor applied to third parameter (Par3 or ParK).  Defaults
        to 1.0.

    Notes
    -----
    Applies a scale factor to parameters used in certain *VXX and *MXX
    operations.   Typical scale factor applications are of the form:

    ParR = FACTR*f(FACT1*Par1)

    or

    ParR = FACTR*((FACT1*Par1) o (FACT2*Par2))

    The factors are applied to each input parameter value before the
    operation and to the result value after the operation.  The scale
    factor settings are reset to the default (1.0) after each *VXX or *MXX
    operation.  Use *VSTAT to list settings.

    This command is valid in any processor.
    """
    command = f"*VFACT,{factr},{fact1},{fact2},{fact3}"
    return self.run(command, **kwargs)


def vfun(self, parr="", func="", par1="", con1="", con2="", con3="",
         **kwargs):
    """Performs a function on a single array parameter.

    APDL Command: *VFUN

    Parameters
    ----------
    parr
        The name of the resulting numeric array parameter vector.  See *SET
        for name restrictions.

    func
        Function to be performed:

        Arccosine: ACOS(Par1). - Arcsine: ASIN(Par1).

        Par1 is sorted in ascending order.  *VCOL, *VMASK, *VCUM, and *VLEN,,NINC do not apply.  *VLEN,NROW does apply. - Arctangent: ATAN(Par1).

        Compress:  Selectively compresses data set.  "True" (*VMASK) values of Par1 (or row positions to be considered according to the NINC value on the *VLEN command) are written in  compressed form to ParR, starting at the specified position. - Copy: Par1 copied to ParR.

        Cosine: COS(Par1). - Hyperbolic cosine: COSH(Par1).

        Direction cosines of the principal stresses (nX9).  Par1 contains the nX6 component stresses for the n locations of the calculations. - Par1 is sorted in descending order.  *VCOL, *VMASK, *VCUM, and *VLEN,,NINC do
                          not apply.  *VLEN,NROW does apply.

        Euler angles of the principal stresses (nX3).  Par1 contains the nX6 component stresses for the n locations of the calculations. - Exponential: EXP(Par1).

        Expand:  Reverse of the COMP function.  All elements of Par1 (starting at the position specified) are written in expanded form to corresponding "true" (*VMASK) positions (or row positions to be considered according to the NINC value on the *VLEN command) of ParR. - Natural logarithm: LOG(Par1).

        Common logarithm: LOG10(Par1). - Nearest integer: 2.783 becomes 3.0, -1.75 becomes -2.0.

        Logical complement: values   0.0 (false) become 1.0 (true).  Values > 0.0 (true) become 0.0 (false). - Principal stresses (nX5). Par1 contains the nX6 component stresses for the n
                          locations of the calculations.

        Power function: Par1**CON1. Exponentiation of any negative number in the vector Par1 to a non-integer power is performed by exponentiating the positive number and prepending the minus sign. For example, -4**2.3 is -(4**2.3). - Sine: SIN(Par1).

        Hyperbolic sine: SINH(Par1). - Square root: SQRT(Par1).

        Tangent: TAN(Par1). - Hyperbolic tangent: TANH(Par1).

        Tangent to a path at a point:  the slope at a point is determined by linear interpolation half way between the previous and next points.  Points are assumed to be in the global Cartesian coordinate system.  Path points are specified in array Par1 (having 3 consecutive columns of data, with the columns containing the x, y, and z coordinate locations, respectively, of the points).  Only the starting row index and the column index for the x coordinates are specified, such as A(1,1).  The y and z coordinates of the vector are assumed to begin in the corresponding next columns, such as A(1,2) and A(1,3).  The tangent result, ParR, must also have 3 consecutive columns of data and will contain the tangent direction vector (normalized to 1.0); such as 1,0,0 for an x-direction vector. - Normal to a path and an input vector at a point: determined from the cross-
                          product of the calculated tangent vector (see
                          TANG) and the input direction vector (with the i,
                          j, and k components input as CON1, CON2, and
                          CON3).  Points are assumed to be in the global
                          Cartesian coordinate system.  Path points are
                          specified in array Par1 (having 3 consecutive
                          columns of data, with the columns containing the
                          x, y, and z coordinate locations, respectively,
                          of the points).  Only the starting row index and
                          the column index for the x coordinates are
                          specified, such as A(1,1).  The y and z
                          coordinates of the vector are assumed to begin in
                          the corresponding next columns, such as A(1,2)
                          and A(1,3).  The normal result, ParR, must also
                          have 3 consecutive columns of data and will
                          contain the normal direction vector (normalized
                          to 1.0); such as 1,0,0 for an x-direction vector.

        Transforms global Cartesian coordinates of a point to the coordinates of a specified system: points to be transformed are specified in array Par1 (having 3 consecutive columns of data, with the columns containing the x, y, and z global Cartesian coordinate locations, respectively, of the points).  Only the starting row index and the column index for the x coordinates are specified, such as A(1,1).  The y and z coordinates of the vector are assumed to begin in the corresponding next columns, such as A(1,2) and A(1,3).  Results are transformed to coordinate system CON1 (which may be any valid coordinate system number, such as 1,2,11,12, etc.).  The transformed result, ParR, must also have 3 consecutive columns of data and will contain the corresponding transformed coordinate locations. - Transforms specified coordinates of a point to global Cartesian coordinates:
                          points to be transformed are specified in array
                          Par1 (having 3 consecutive columns of data, with
                          the columns containing the local coordinate
                          locations (x, y, z or r, θ, z or etc.) of the
                          points).  Only the starting row index and the
                          column index for the x coordinates are specified,
                          such as A(1,1).  The y and z coordinates (or θ
                          and z, or etc.) of the vector are assumed to
                          begin in the corresponding next columns, such as
                          A(1,2) and A(1,3).  Local coordinate locations
                          are assumed to be in coordinate system CON1
                          (which may be any valid coordinate system number,
                          such as 1,2,11,12, etc.).  The transformed
                          result, ParR, must also have 3 consecutive
                          columns of data, with the columns containing the
                          global Cartesian x, y, and z coordinate
                          locations, respectively.

    par1
        Array parameter vector in the operation.

    con1, con2, con3
        Constants (used only with the PWR, NORM, LOCAL, and GLOBAL
        functions).

    Notes
    -----
    Operates on one input array parameter vector and produces one output
    array parameter vector according to:

    ParR = f(Par1)

    where the functions (f) are described below.  Functions are based on
    the standard FORTRAN definitions where possible.  Out-of-range function
    results (or results with exponents whose magnitudes are approximately
    greater than 32 or less than -32) produce a zero value.  Input and
    output for angular functions may be radians (default) or degrees
    [*AFUN].  ParR may be the same as Par1.  Starting array element numbers
    must be defined for each array parameter vector if it does not start at
    the first location. For example, *VFUN,A,SQRT,B(5) takes the square
    root of the fifth element of B and stores the result in the first
    element of A.  Operations continue on successive array elements [*VLEN,
    *VMASK] with the default being all successive elements.  Absolute
    values and scale factors may be applied to all parameters [*VABS,
    *VFACT].  Results may be cumulative [*VCUM].  Skipping array elements
    via *VMASK or *VLEN for the TANG and NORM functions skips only the
    writing of the results (skipped array element data are used in all
    calculations).  See the *VOPER command for detail   s   .   /   p   >
    p   >   T   h   i   s       c   o   m   m   a   n   d       i   s
    v   a   l   i   d       i   n       a   n   y       p   r   o   c   e
    s   s   o   r   .   /   p   >   /   d   i   v   >   d   i   v       c
    l   a   s   s   =   "   r   e   f   s   e   c   t   1   "       t   i
    t   l   e   =   "   M   e   n   u       P   a   t   h   s   "   >   a
    n   a   m   e   =   "   d   0   e   2   9   2   8   5   8   "   >   /
    a   >   h   2   >   M   e   n   u       P   a   t   h   s   /   h   2
    >   t   a   b   l   e       b   o   r   d   e   r   =   "   0   "
    s   u   m   m   a   r   y   =   "   S   i   m   p   l   e       l   i
    s   t   "       c   l   a   s   s   =   "   s   i   m   p   l   e   l
    i   s   t   "   >   t   r   >   t   d   >   s   p   a   n       c   l
    a   s   s   =   "   g   u   i   m   e   n   u   "   >   s   t   r   o
    n   g   >   U   t   i   l   i   t   y       M   e   n   u   &g   t   ;
    P   a   r   a   m   e   t   e   r   s   &g   t   ;   A   r   r   a   y
    O   p   e   r   a   t   i   o   n   s   &g   t   ;   V   e   c   t   o
    r       F   u   n   c   t   i   o   n   s   /   s   t   r   o   n   g
    >   /   s   p   a   n   >   /   t   d   >   /   t   r   >   /   t   a
    b   l   e   >   /   d   i   v   >   /   d   i   v   >   h   r   >   p
    c   l   a   s   s   =   "   l   e   g   a   l   f   o   o   t   e   r
    "   >   s   m   a   l   l   >   i   >   R   e   l   e   a   s   e
    1   6   .   2       -       &c   o   p   y   ;       S   A   S       I
    P   ,       I   n   c   .       A   l   l       r   i   g   h   t   s
    r   e   s   e   r   v   e   d   .   /   i   >   /   s   m   a   l   l
    >   /   p   >   /   b   o   d   y   >   /   h   t   m   l   >
    """
    command = f"*VFUN,{parr},{func},{par1},{con1},{con2},{con3}"
    return self.run(command, **kwargs)


def vitrp(self, parr="", part="", pari="", parj="", park="", **kwargs):
    """Forms an array parameter by interpolation of a table.

    APDL Command: *VITRP

    Parameters
    ----------
    parr
        The name of the resulting array parameter.  See *SET for name
        restrictions.

    part
        The name of the TABLE array parameter.  The parameter must exist as
        a dimensioned array of type TABLE [*DIM].

    pari
        Array parameter vector of I (row) index values for interpolation in
        ParT.

    parj
        Array parameter vector of J (column) index values for interpolation
        in ParT (which must be at least 2-D).

    park
        Array parameter vector of K (depth) index values for interpolation
        in ParT (which must be 3-D).

    Notes
    -----
    Forms an array parameter (of type ARRAY) by interpolating values of an
    array parameter (of type TABLE) at specified table index locations
    according to:

    ParR = f(ParT, Parl, ParJ, ParK)

    where ParT is the type TABLE array parameter, and ParI, ParJ, ParK are
    the type ARRAY array parameter vectors of index values for
    interpolation in ParT.  See the *DIM command for TABLE and ARRAY
    declaration types.  Linear interpolation is used.  The starting array
    element number for the TABLE array (ParT) is not used (but a value must
    be input).  Starting array element numbers must be defined for each
    array parameter vector if it does not start at the first location. For
    example, *VITRP,R(5),TAB(1,1),X(2),Y(4) uses the second element of X
    and the fourth element of Y as index values (row and column) for a 2-D
    interpolation in TAB and stores the result in the fifth element of R.
    Operations continue on successive array elements [*VLEN, *VMASK] with
    the default being all successive elements.  Absolute values and scale
    factors may be applied to the result parameter [*VABS, *VFACT].
    Results may be cumulative [*VCUM].  See the *VOPER command for details.

    This command is valid in any processor.
    """
    command = f"*VITRP,{parr},{part},{pari},{parj},{park}"
    return self.run(command, **kwargs)


def vlen(self, nrow="", ninc="", **kwargs):
    """Specifies the number of rows to be used in array parameter operations.

    APDL Command: *VLEN

    Parameters
    ----------
    nrow
        Number of rows to be used with the *VXX or *MXX operations.
        Defaults to the number of rows needed to fill the result array.

    ninc
        Perform the operation on every NINC row (defaults to 1).

    Notes
    -----
    Specifies the number of rows to be used in array parameter operations.
    The size of the submatrix used is determined from the upper left
    starting array element (defined on the operation command) to the lower
    right array element (defined by the number of rows on this command and
    the number of columns on the *VCOL command).  NINC allows skipping row
    operations for some operation commands.  Skipped rows are included in
    the row count.  The starting row number must be defined on the
    operation command for each parameter read and for the result written.

    The default NROW is calculated from the maximum number of rows of the
    result array (the *DIM row dimension) minus the starting location + 1.
    For example, *DIM,R,,10 and a starting location of R(7) gives a default
    of 4 loops (filling R(7), R(8), R(9), and R(10)).  Repeat operations
    automatically terminate at the last row of the result array.  Existing
    values in the rows and columns of the results matrix remain unchanged
    where not overwritten by the requested input or operation values.

    The stride (NINC) allows operations to be performed at regular
    intervals.  It has no effect on the total number of row operations.
    Skipped operations retain the previous result.  For example, *DIM,R,,6,
    with a starting location of R(1), NROW = 10, and NINC = 2 calculates
    values for locations R(1), R(3), and R(5) and retains values for
    locations R(2), R(4), and R(6).  A more general skip control may be
    done by masking [*VMASK].  The row control settings are reset to the
    defaults after each *VXX or *MXX operation.  Use *VSTAT to list
    settings.

    This command is valid in any processor.
    """
    command = f"*VLEN,{nrow},{ninc}"
    return self.run(command, **kwargs)


def vmask(self, par="", **kwargs):
    """Specifies an array parameter as a masking vector.

    APDL Command: *VMASK

    Parameters
    ----------
    par
        Name of the mask parameter.  The starting subscript must also be
        specified.

    Notes
    -----
    Specifies the name of the parameter whose values are to be checked for
    each resulting row operation.  The mask vector usually contains only 0
    (for false) and 1 (for true) values.  For each row operation the
    corresponding mask vector value is checked.  A true value allows the
    operation to be done.  A false value skips the operation (and retains
    the previous results).  A mask vector can be created from direct input,
    such as M(1) = 1,0,0,1,1,0,1; or from the DATA function of the *VFILL
    command.  The NOT function of the *VFUN command can be used to reverse
    the logical sense of the mask vector.  The logical compare operations
    (LT, LE, EQ, NE, GE, and GT) of the *VOPER command also produce a mask
    vector by operating on two other vectors.  Any numeric vector can be
    used as a mask vector since the actual interpretation assumes values
    less than 0.0 are 0.0 (false) and values greater than 0.0 are 1.0
    (true).  If the mask vector is not specified (or has fewer values than
    the result vector), true (1.0) values are assumed for the unspecified
    values.  Another skip control may be input with NINC on the *VLEN
    command.  If both are present, operations occur only when both are
    true.  The mask setting is reset to the default (no mask) after each
    *VXX or *MXX operation.  Use *VSTAT to list settings.

    This command is valid in any processor.
    """
    command = f"*VMASK,{par}"
    return self.run(command, **kwargs)


def voper(self, parr="", par1="", oper="", par2="", con1="", con2="",
          **kwargs):
    """Operates on two array parameters.

    APDL Command: *VOPER

    Parameters
    ----------
    parr
        The name of the resulting array parameter vector.  See *SET  for
        name restrictions.

    par1
        First array parameter vector in the operation.  May also be a
        scalar parameter or a literal constant.

    oper
        Operations:

        Addition: Par1+Par2. - Subtraction: Par1-Par2.

        Multiplication: Par1*Par2. - Division: Par1/Par2  (a divide by zero results in a value of zero).

        Minimum: minimum of Par1 and Par2. - Maximum: maximum of Par1 and Par2.

        Less than comparison: Par1<Par2 gives 1.0 if true, 0.0 if false. - Less than or equal comparison: Par1 Par2 gives 1.0 if true, 0.0 if false.

        Equal comparison: Par1 = Par2 gives 1.0 if true, 0.0 if false. - Not equal comparison: Par1 ≠ Par2 gives 1.0 if true, 0.0 if false.

        Greater than or equal comparison: Par1 Par2 gives 1.0 if true, 0.0 if false. - Greater than comparison: Par1>Par2 gives 1.0 if true, 0.0 if false.

        First derivative: d(Par1)/d(Par2).  The derivative at a point is determined  over points half way between the previous and next points (by linear  interpolation). Par1 must be a function (a unique Par1 value for each Par2 value) and Par2 must be in ascending order. - Second derivative: d2(Par1)/d(Par2)2.  See also DER1.

        Single integral:    Par1 d(Par2), where CON1 is the integration constant.  The integral at a point is determined by using the single integration procedure described in the Mechanical APDL Theory Reference. - Double integral:       Par1 d(Par2), where CON1 is the integration constant of
                          the first integral and CON2 is the integration
                          constant of the second integral.  If Par1
                          contains acceleration data, CON1 is the initial
                          velocity and CON2 is the initial displacement.
                          See also INT1.

        Dot product: Par1  . Par2.  Par1 and Par2 must each have three consecutive columns of data, with the columns containing the i, j, and k vector components, respectively.  Only the starting row index and the column index for the i components are specified for Par1 and Par2, such as A(1,1).  The j and k components of the vector are assumed to begin in the corresponding next columns, such as A(1,2) and A(1,3). - Cross product: Par1 x Par2.  Par1, Par2, and ParR must each have 3 components,
                          respectively.  Only the starting row index and
                          the column index for the i components are
                          specified for Par1, Par2, and ParR, such as
                          A(1,1).  The j and k components of the vector are
                          assumed to begin in the corresponding next
                          columns, such as A(1,2) and A(1,3).

        Gather:  For a vector of position numbers, Par2, copy the value of Par1 at each position number to ParR.  Example:  for Par1 = 10,20,30,40 and Par2 = 2,4,1;  ParR = 20,40,10. - Scatter:  Opposite of GATH operation.  For a vector of position numbers, Par2,
                          copy the value of Par1 to that position number in
                          ParR.  Example: for Par1 = 10,20,30,40,50 and
                          Par2 = 2,1,0,5,3; ParR = 20,10,50,0,40.

        Arctangent: arctangent of Par1/Par2 with the sign of each component considered. - Transform the data in Par1 from the global Cartesian coordinate system to the
                          local coordinate system given in CON1. Par1 must
                          be an N x 3 (i.e., vector) or an N x 6 (i.e.,
                          stress or strain tensor) array. If the local
                          coordinate system is a cylindrical, spherical, or
                          toroidal system, then you must provide the global
                          Cartesian coordinates in Par2 as an N x 3 array.
                          Set CON2 = 1 if the data is strain data.

    par2
        Second array parameter vector in the operation.  May also be a
        scalar parameter or a literal constant.

    con1
        First constant (used only with the INT1 and INT2 operations).

    con2
        Second constant (used only with the INT2 operation).

    Notes
    -----
    Operates on two input array parameter vectors and produces one output
    array parameter vector according to:

    ParR = Par1 o Par2

    where the operations (o) are described below.  ParR may be the same as
    Par1 or Par2.  Absolute values and scale factors may be applied to all
    parameters [*VABS, *VFACT].  Results may be cumulative [*VCUM].
    Starting array element numbers must be defined for each array parameter
    vector if it does not start at the first location, such as
    *VOPER,A,B(5),ADD,C(3) which adds the third element of C to the fifth
    element of B and stores the result in the first element of A.
    Operations continue on successive array elements [*VLEN, *VMASK] with
    the default being all successive elements.  Skipping array elements via
    *VMASK or *VLEN for the DER_ and INT_ functions skips only the writing
    of the results (skipped array element data are used in all
    calculations).

    Parameter functions and operations are available to operate on a scalar
    parameter or a single element of an array parameter, such as SQRT(B) or
    SQRT(A(4)).  See the *SET command for details.  Operations on a
    sequence of array elements can be done by repeating the desired
    function or operation in a do-loop [*DO].  The vector operations within
    the ANSYS program (*VXX commands) are internally programmed do-loops
    that conveniently perform the indicated operation over a sequence of
    array elements.  If the array is multidimensional, only the first
    subscript is incremented in the do-loop, that is, the operation repeats
    in column vector fashion "down" the array.  For example, for A(1,5),
    A(2,5), A(3,5), etc.  The starting location of the row index must be
    defined for each parameter read and for the result written.

    The default number of loops is from the starting result location to the
    last result location and can be altered with the *VLEN command.  A
    logical mask vector may be defined to control at which locations the
    operations are to be skipped [*VMASK].  The default is to skip no
    locations.  Repeat operations automatically terminate at the last array
    element of the result array column if the number of loops is undefined
    or if it exceeds the last result array element.  Zeroes are used in
    operations for values read beyond the last array element of an input
    array column.  Existing values in the rows and columns of the results
    matrix
    """
    command = f"*VOPER,{parr},{par1},{oper},{par2},{con1},{con2}"
    return self.run(command, **kwargs)




def vscfun(self, parr="", func="", par1="", **kwargs):
    """Determines properties of an array parameter.

    APDL Command: *VSCFUN

    Parameters
    ----------
    parr
        The name of the resulting scalar parameter.  See *SET for name
        restrictions.

    func
        Functions:

        Maximum: the maximum Par1 array element value. - Minimum: the minimum Par1 array element value.

        Index location of the maximum Par1 array element value.  Array Par1 is searched starting from its specified index. - Index location of the minimum Par1 array element value.  Array Par1 is searched
                          starting from its specified index.

        Index location of the first nonzero value in array Par1.  Array Par1 is searched starting from its specified index. - Index location of the last nonzero value in array Par1.  Array Par1 is searched
                          starting from its specified index.

        Sum:  Par1 (the summation of the Par1 array element values). - Median: value of Par1 at which there are an equal number of values above and
                          below.

        Mean: (σ Par1)/NUM, where NUM is the number of summed values. - Variance: (σ ((Par1-MEAN)**2))/NUM.

        Standard deviation: square root of VARI. - Root-mean-square: square root of (σ (Par1**2))/NUM.

    par1
        Array parameter vector in the operation.

    Notes
    -----
    Operates on one input array parameter vector and produces one output
    scalar parameter according to:

    ParR = f(Par1)

    where the functions (f) are described below. The starting array element
    number must be defined for the array parameter vector.  For example,
    *VSCFUN,MU,MEAN,A(1) finds the mean of the A vector values, starting
    from the first value and stores the result as parameter MU.  Operations
    use successive array elements [*VLEN, *VMASK] with the default being
    all successive array elements.  Absolute values and scale factors may
    be applied to all parameters [*VABS, *VFACT].  Results may be
    cumulative [*VCUM].  See the *VOPER command for details.

    This command is valid in any processor.
    """
    command = f"*VSCFUN,{parr},{func},{par1}"
    return self.run(command, **kwargs)


def vstat(self, **kwargs):
    """Lists the current specifications for the array parameters.

    APDL Command: *VSTAT

    Notes
    -----
    Lists the current specifications for the *VABS, *VCOL, *VCUM, *VFACT,
    *VLEN, and *VMASK commands.

    This command is valid in any processor.
    """
    command = f"*VSTAT,"
    return self.run(command, **kwargs)


def vwrite(self, par1="", par2="", par3="", par4="", par5="", par6="",
           par7="", par8="", par9="", par10="", par11="", par12="",
           par13="", par14="", par15="", par16="", par17="", par18="",
           par19="", **kwargs):
    """Writes data to a file in a formatted sequence.

    APDL Command: *VWRITE

    Parameters
    ----------
    par1, par2, par3, . . . , par19
        You can write up to 19 parameters (or constants) at a time. Any Par
        values after a blank  Par value are ignored.  If you leave them all
        blank, one line will be written (to write a title or a blank line).
        If you input the keyword SEQU, a sequence of numbers (starting from
        1) will be written for that item.

    Notes
    -----
    You use *VWRITE to write data to a file in a formatted sequence. Data
    items (Par1, Par2, etc.) may be array parameters, scalar parameters,
    character parameters (scalar or array), or constants.  You must
    evaluate expressions and functions in the data item fields before using
    the *VWRITE command, since initially they will be evaluated to a
    constant and remain constant throughout the operation.  Unless a file
    is defined with the *CFOPEN  command, data is written to the standard
    output file. Data written to the standard output file may be diverted
    to a different file by first switching the current output file with the
    /OUTPUT command. You can also use the *MWRITE command to write data to
    a specified file. Both commands contain format descriptors on the line
    immediately following the command. The format descriptors can be in
    either Fortran or C format.

    You must enclose Fortran format descriptors in parentheses. They must
    immediately follow the *VWRITE command on a separate line of the same
    input file.  Do not include the word FORMAT. The format must specify
    the number of fields to be written per line, the field width, the
    placement of the decimal point, etc.  You should use one field
    descriptor for each data item written.  The write operation uses your
    system's available FORTRAN FORMAT conventions (see your system FORTRAN
    manual).  You can use any standard FORTRAN real format (such as
    (4F6.0), (E10.3,2X,D8.2), etc.) and alphanumeric format (A).
    Alphanumeric strings are limited to a maximum of 8 characters for any
    field (A8) using the Fortran format. Use the "C" format for string
    arrays larger than 8 characters. Integer (I) and list-directed (*)
    descriptors may not be used.  You can include text in the format as a
    quoted string.  The parentheses must be included in the format and the
    format must not exceed 80 characters (including parentheses).  The
    output line length is limited to 128 characters.

    The "C" format descriptors are used if the first character of the
    format descriptor line is not a left parenthesis. "C" format
    descriptors are up to 80 characters long, consisting of text strings
    and predefined "data descriptors" between the strings where numeric or
    alphanumeric character data will be inserted.  The normal descriptors
    are %I for integer data, %G for double precision data, %C for
    alphanumeric character data, and %/ for a line break. There must be one
    data descriptor for each specified value (8 maximum) in the order of
    the specified values. The enhanced formats described in *MSG may also
    be used.

    For array parameter items, you must define the starting array element
    number. Looping continues (incrementing the vector index number of each
    array parameter by one) each time you output a line, until the maximum
    array vector element is written.  For example, *VWRITE,A(1) followed by
    (F6.0) will write one value per output line, i.e., A(1), A(2), A(3),
    A(4), etc.  You write constants and scalar parameters with the same
    values for each loop.  You can also control the number of loops and
    loop skipping with the *VLEN and *VMASK commands.  The vector
    specifications *VABS,  *VFACT, and *VCUM  do not apply to this command.
    If looping continues beyond the supplied data array's length, zeros
    will be output for numeric array parameters and blanks for character
    array parameters.  For multi-dimensioned array parameters, only the
    first (row) subscript is incremented.  See the *VOPER command for
    details.  If you are in the GUI, the *VWRITE command must be contained
    in an externally prepared file and read into ANSYS (i.e., *USE, /INPUT,
    etc.).

    This command is valid in any processor.
    """
    # cannot be in interactive mode
    if not self._store_commands:
        raise RuntimeError('VWRTIE cannot run interactively.  \n\nPlease use '
                           '``with mapdl.non_interactive:``')

    command = f"*VWRITE,{par1},{par2},{par3},{par4},{par5},{par6},{par7},{par8},{par9},{par10},{par11},{par12},{par13},{par14},{par15},{par16},{par17},{par18},{par19}"
    return self.run(command, **kwargs)



def dump(self, nstrt="", nstop="", **kwargs):
    """Dumps the contents of a binary file.

    APDL Command: DUMP

    Parameters
    ----------
    nstrt, nstop
        Dump file from record NSTRT (defaults to 1) to NSTOP (defaults to
        NSTRT).  If NSTRT = HEAD, dump only record 1 of the file (NSTOP and
        the format specification are ignored).  If NSTRT = ALL, dump the
        entire file.

    Notes
    -----
    Dumps the file named on the AUX2 FILEAUX2 command according the format
    specified on the FORM command.
    """
    command = f"DUMP,{nstrt},{nstop}"
    return self.run(command, **kwargs)


def fileaux2(self, fname="", ident="", **kwargs):
    """Specifies the binary file to be dumped.

    APDL Command: FILEAUX2

    Parameters
    ----------
    fname
        File name and directory path (248 characters maximum,
        including the characters needed for the directory path).
        An unspecified directory path defaults to the working
        directory; in this case, you can use all 248 characters
        for the file name.

    ident
        ANSYS filename identifier.  See the Basic Analysis Guide
        for file descriptions and identifiers.  If not an ANSYS
        identifier, Ident will be used as the filename extension.

    Notes
    -----
    Specifies the binary file to be dumped with the DUMP command.
    """
    command = f"FILEAUX2,{fname},{ident}"
    return self.run(command, **kwargs)


def form(self, lab="", **kwargs):
    """Specifies the format of the file dump.

    APDL Command: FORM

    Parameters
    ----------
    lab
        Format:

        RECO - Basic record description only (minimum output) (default).

        TEN - Same as RECO plus the first ten words of each record.

        LONG - Same as RECO plus all words of each record.

    Notes
    -----
    Specifies the format of the file dump (from the DUMP command).
    """
    command = f"FORM,{lab}"
    return self.run(command, **kwargs)


def ptr(self, loc="", base="", **kwargs):
    """Dumps the record of a binary file.

    APDL Command: PTR

    Parameters
    ----------
    loc, base
        Dump the file record starting at pointer LOC. BASE is the base
        pointer, and would be used if LOC is a relative pointer.

    Notes
    -----
    Dumps the record of the file named on the AUX2 FILEAUX2 command
    according the format specified on the FORM command.
    """
    command = f"PTR,{loc},{base}"
    return self.run(command, **kwargs)
